@@ -29,6 +29,11 @@
  * email: Ulrich dot Raich at cern dot ch
  */
 
+/*
+ * xerr messages Copyright (C) 1989-2021  Alan R. Baldwin
+ * from ASxxxx 5.40
+ */
+
 /*
  * Extensions: P. Felber
  *
@@ -116,7 +121,7 @@ VOID  machine(struct mne * mp)
                         if ((v1 = admode(CND)) != 0) {
                                 outab(op | (v1<<3));
                         } else {
-                                qerr();
+                                xerr('a', "Condition code required.");
                         }
                 } else {
                         outab(0xC9);
@@ -153,7 +158,7 @@ VOID  machine(struct mne * mp)
                                 break;
                         }
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_RST:
@@ -164,11 +169,11 @@ VOID  machine(struct mne * mp)
                  *   rabbit processor
                  */
                 if ((v1 == 0x00) || (v1 == 0x08) || (v1 == 0x30)) {
-                        aerr( );
+                        xerr('a', "Rabbit 2000/4000: 0x00, 0x08, and 0x30 are not allowed");
                         v1 = 0;
                 }
                 if (v1 & ~0x38) {
-                        aerr();
+                        xerr('a', "Allowed values: N * 0x08, N = 0 -> 7."); 
                         v1 = 0;
                 }
                 outab(op|v1);
@@ -180,7 +185,7 @@ VOID  machine(struct mne * mp)
                 expr(&e1, 0);
                 abscheck(&e1);
                 if (e1.e_addr > 3) {
-                        aerr();
+                        xerr('a', "Values of 0, 1, 2, and 3 are valid.");
                         e1.e_addr = 0;
                 }
                 outab(op);
@@ -200,7 +205,7 @@ VOID  machine(struct mne * mp)
                 addr(&e2);
                 abscheck(&e1);
                 if (genop(0xCB, op, &e2, 0) || t1)
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_RL:
@@ -331,7 +336,7 @@ VOID  machine(struct mne * mp)
                         /* sbc  hl, [bc|de|hl|sp] */
                         if ( v2 != gixiy(v2) )
                                 /* sorry, sbc hl, [ix|iy] do not exist */
-                                aerr( );
+                                xerr('a', "Second argument: must be BC, DE, HL or SP.");
 
                         outab(0xED);
                         outab(0x42 | v2 << 4);
@@ -346,7 +351,7 @@ VOID  machine(struct mne * mp)
                         /*  EC     or   hl, de */
                         /*  ED 48  cp   hl, de */
                         if (rf == S_SBC) /* op == 0x98 */
-                                aerr( );
+                                xerr('a', "Not valid for SBC.");
 
                         switch( op ) {
                         case 0x90:  /* sub */ outab(0x55); break;
@@ -377,7 +382,7 @@ VOID  machine(struct mne * mp)
                         /* ED F6   or   jkhl, bcde  */
                         /* ED 58   cp   jkhl, bcde  */
                         if (rf == S_SBC) /* op == 0x98 */
-                                aerr( );
+                                xerr('a', "Not valid for SBC.");
 
                         outab(0xED);
                         switch( op ) {
@@ -397,7 +402,7 @@ VOID  machine(struct mne * mp)
                         break;
                 }
 
-                aerr( );
+                xerr('a', "Not valid for SBC.");
                 break;
 
         case S_ADD:
@@ -421,7 +426,7 @@ VOID  machine(struct mne * mp)
                         }
 
                         if (genop(0, op, &e1, 1))
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         break;
                 }
                 if ((t1 == S_R8) && (e1.e_addr == A)) {
@@ -430,7 +435,7 @@ VOID  machine(struct mne * mp)
                                 outab(0x7F);
 
                         if (genop(0, op, &e2, 1))
-                                aerr();
+                                xerr('a', "Second argument: Invalid Addressing Mode.");
                         break;
                 }
 
@@ -478,7 +483,7 @@ VOID  machine(struct mne * mp)
                         outab(0x09 | (v2 << 4));
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_LD:
@@ -499,7 +504,7 @@ VOID  machine(struct mne * mp)
                                 /* exception for "ld a,a" 
                                  * on rabbit 4000 0x7F is a prefix instead of "ld a,a"
                                  */
-                                aerr( );
+                                xerr('a', "Not A Rabbit 4000 Instruction");
                         }
 
                         if ((v1 == A) && (t2 == S_R8)) {
@@ -785,7 +790,7 @@ VOID  machine(struct mne * mp)
                                 break;
                         }
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
       
         case S_EX:
@@ -806,11 +811,16 @@ VOID  machine(struct mne * mp)
                                 }
                         }
         
-                        if ((t1 == S_IDSP) && (v1 == 0) && (v2!=HL)) {
+                        if ((t1 == S_IDSP) && (v1 == 0)) {
                                 /* 0xE3 is EX DE',HL on rabbit 2000 
                                  * but DD/FD E3 "ex (sp),ix|iy" is valid
                                  */
-                                if (gixiy(v2) == HL) {
+                                if (v2 == HL) {
+                                        outab(0xED);
+                                        outab(0x54);
+                                        break;
+                                }
+                                else if (gixiy(v2) == HL) {
                                         outab(op);
                                         break;
                                 }
@@ -824,7 +834,7 @@ VOID  machine(struct mne * mp)
                         outab(0xB4);
                         break;
                 }
-                aerr();
+		xerr('a', "Invalid Addressing Mode.");
                 break;
       
         case S_IN:
@@ -860,7 +870,7 @@ VOID  machine(struct mne * mp)
                                 break;
                         }
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
       
         case S_NEG:
@@ -898,7 +908,7 @@ VOID  machine(struct mne * mp)
                         if ((v1 &= 0xFF) <= 0x03) {
                                 op += (v1+1)<<3;
                         } else {
-                                aerr();
+                                xerr('a', "Condition code required.");
                         }
                         comma(1);
                 }
@@ -907,7 +917,7 @@ VOID  machine(struct mne * mp)
                 if (mchpcr(&e2)) {
                         v2 = (int) (e2.e_addr - dot.s_addr - 1);
                         if (pass == 2 && ((v2 < -128) || (v2 > 127)))
-                                aerr();
+                                xerr('a', "Branching Range Exceeded.");
                         outab(v2);
                 } else {
                         outrb(&e2, R_PCR);
@@ -942,7 +952,7 @@ VOID  machine(struct mne * mp)
                         outab(0xE9);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 /*
     case X_HD64:
@@ -950,12 +960,16 @@ VOID  machine(struct mne * mp)
       break;
 */
         case HD_INH2:
+                if (mchtyp != X_HD64)
+                        xerr('a', "A Z180 Instruction.");
                 outab(0xED);
                 outab(op);
                 break;
       
         case HD_IN:
         case HD_OUT:
+                if (mchtyp != X_HD64)
+                        xerr('a', "A Z180 Instruction.");
                 if (rf == HD_IN) {
                         t1 = addr(&e1);
                         comma(1);
@@ -971,20 +985,24 @@ VOID  machine(struct mne * mp)
                         outrb(&e2, 0);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
       
         case HD_MLT:
+                if (mchtyp != X_HD64)
+                        xerr('a', "A Z180 Instruction.");
                 t1 = addr(&e1);
                 if ((t1 == S_R16) && ((v1 = (int) e1.e_addr) <= SP)) {
                         outab(0xED);
                         outab(op | (v1<<4));
                         break;
                 }
-                aerr();
+                xerr('a', "Only BC, DE, HL and SP are allowed.");
                 break;
       
         case HD_TST:
+                if (mchtyp != X_HD64)
+                        xerr('a', "A Z180 Instruction.");
                 t1 = addr(&e1);
                 if (t1 == S_R8) {
                         outab(0xED);
@@ -1002,10 +1020,12 @@ VOID  machine(struct mne * mp)
                         outrb(&e1, 0);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
       
         case HD_TSTIO:
+                if (mchtyp != X_HD64)
+                        xerr('a', "A Z180 Instruction.");
                 t1 = addr(&e1);
                 if (t1 == S_IMMED) {
                         outab(0xED);
@@ -1013,7 +1033,7 @@ VOID  machine(struct mne * mp)
                         outrb(&e1, 0);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case X_LJP:
@@ -1039,19 +1059,19 @@ VOID  machine(struct mne * mp)
                         outab(op);
                         break;
                 }
-                aerr( );
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case R3K_INH1:
                 if (!(r3k_mode || r4k_mode))
-                        err('o');
+                        xerr('o', "A Rabbit 3000/4000 Instruction.");
       
                 outab(op);
                 break;
       
         case R3K_INH2:
                 if (!(r3k_mode || r4k_mode))
-                        err('o');
+                        xerr('o', "A Rabbit 3000/4000 Instruction.");
       
                 outab(0xED);
                 outab(op);
@@ -1059,7 +1079,7 @@ VOID  machine(struct mne * mp)
 
         case R4K_INH2:
                 if (!r4k_mode)
-                        err('o');
+                        xerr('o', "A Rabbit 4000 Instruction.");
       
                 outab(0xED);
                 outab(op);
@@ -1067,14 +1087,14 @@ VOID  machine(struct mne * mp)
       
         case X_R4K_MULU:
                 if (!r4k_mode)
-                        err('o');
+                        xerr('o', "A Rabbit 4000 Instruction.");
       
                 outab(op);
                 break;
       
         case X_JRE:
                 if (!r4k_mode)
-                        err('o');
+                        xerr('o', "A Rabbit 4000 Instruction.");
       
                 if ((v1 = admode(ALT_CND)) != 0) {
                         op += v1<<3;
@@ -1099,7 +1119,7 @@ VOID  machine(struct mne * mp)
       
         case X_CLR:
                 if (!r4k_mode)
-                        err('o');
+                        xerr('o', "A Rabbit 4000 Instruction.");
                 t1 = addr(&e1);
                 v1 = (int) e1.e_addr;
                 if ((t1 == S_R16) && (v1 == HL)) {
@@ -1110,7 +1130,8 @@ VOID  machine(struct mne * mp)
                 break;
 
         default:
-                err('o');
+                xerr('o', "Internal Opcode Error.");
+                break;
         }
 }
 
@@ -1137,7 +1158,7 @@ genop(int pop, int op, struct expr *esp, int f)
          */
         if (t1 == S_IDHL) {
                 if ((esp->e_base.e_ap != NULL) || (esp->e_addr != 0))
-                        aerr();
+                        xerr('a', "(HL+D) is invalid.");
                 if (pop)
                         outab(pop);
                 outab(op|0x06);
