@@ -57,6 +57,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "cmdlexcl.h"
 
 bool jaj= false;
+int juj= 0;
 
 
 /*
@@ -69,6 +70,14 @@ cl_app::cl_app(void)
   sim= 0;
   in_files= new cl_ustrings(2, 2, "input files");
   options= new cl_options();
+  quiet= false;
+  if (app_start_at == 0)
+    app_start_at= dnow();
+  srnd(0);
+  set_console_mode();
+  period= 0;
+  cyc= 0;
+  acyc= 0;
 }
 
 cl_app::~cl_app(void)
@@ -76,6 +85,7 @@ cl_app::~cl_app(void)
   remove_simulator();
   delete commander;
   delete in_files;
+  delete ocon;
   delete options;
 }
 
@@ -84,12 +94,15 @@ cl_app::init(int argc, char *argv[])
 {
   sigpipe_off();
   cl_base::init();
-  set_name(cchars("application"));
+  if (!have_real_name())
+    set_name("application");
   mk_options();
   proc_arguments(argc, argv);
   class cl_cmdset *cmdset= new cl_cmdset();
   cmdset->init();
   build_cmdset(cmdset);
+  ocon= new cl_console_stdout(this);
+  ocon->init();
   commander= new cl_commander(this, cmdset/*, sim*/);
   commander->init();
   return(0);
@@ -108,12 +121,12 @@ int
 cl_app::run(void)
 {
   int done= 0;
-  double input_last_checked= 0;
+  //double input_last_checked= 0;
   class cl_option *o= options->get_option("go");
   bool g_opt= false;
-  unsigned int cyc= 0;
   enum run_states rs= rs_config;
-    
+
+  cperiod.set(cperiod_value());
   while (!done)
     {
       if ((rs == rs_config) &&
@@ -128,11 +141,11 @@ cl_app::run(void)
 	      int i;
 	      for (i= 0; i < in_files->count; i++)
 		{
-		  char *fname= (char *)(in_files->at(i));
+		  const char *fname= (const char *)(in_files->at(i));
 		  long l;
 		  if ((l= sim->uc->read_file(fname, NULL)) >= 0)
 		    {
-		      ///*commander->all_printf*/printf("%ld words read from %s\n", l, fname);
+		      sim->uc->reset();
 		    }
 		}
 	    }
@@ -146,7 +159,6 @@ cl_app::run(void)
 	    sim->start(0, 0);
 	  rs= rs_run;
 	}
-      ++cyc;
       if (!sim)
 	{
 	  commander->wait_input();
@@ -154,21 +166,23 @@ cl_app::run(void)
 	}
       else
         {
+	  acyc++;
           if (sim->state & SIM_GO)
             {
-	      if (cyc - input_last_checked > 10000)
+	      if (++cyc > period)
 		{
-		  input_last_checked= cyc;
+		  cyc= 0;
 		  if (sim->uc)
 		    sim->uc->touch();
 		  if (commander->input_avail())
 		    done= commander->proc_input();
 		}
 	      sim->step();
-	      if (jaj && commander->frozen_console)
+	      if (jaj) ocon->dd_printf("** %u\n",MU(acyc));
+	      if (jaj && commander->frozen_or_actual())
 		{
-		  sim->uc->print_regs(commander->frozen_console),
-		    commander->frozen_console->dd_printf("\n");
+		  sim->uc->print_regs(commander->frozen_or_actual()),
+		    commander->frozen_or_actual()->dd_printf("\n");
 		}
             }
 	  else
@@ -200,14 +214,14 @@ cl_app::done(void)
  */
 
 static void
-print_help(char *name)
+print_help(const char *name)
 {
   printf("%s: %s\n", name, VERSIONSTR);
-  printf("Usage: %s [-hHVvPgGw] [-p prompt] [-t CPU] [-X freq[k|M]]\n"
-	 "       [-C cfg_file] [-c file] [-s file] [-S optionlist]\n"
-	 "       [-a nr]"
+  printf("Usage: %s [-hHVvPgGwlbBq] [-p prompt] [-t CPU] [-X freq[k|M]] [-R seed]\n"
+	 "       [-C cfg_file] [-c file] [-e command] [-s file] [-S optionlist]\n"
+	 "       [-I if_optionlist] [-o colorlist] [-a nr]\n"
 #ifdef SOCKET_AVAIL
-	 " [-Z portnum] [-k portnum]"
+	 "       [-Z portnum] [-k portnum]"
 #endif
 	 "\n"
 	 "       [files...]\n", name);
@@ -216,7 +230,9 @@ print_help(char *name)
      "Options:\n"
      "  -t CPU       Type of CPU: 51, C52, 251, etc.\n"
      "  -X freq[k|M] XTAL frequency\n"
+     "  -R seed      Set the random number generator seed value\n"
      "  -C cfg_file  Read initial commands from `cfg_file' and execute them\n"
+     "  -e command   Execute command on startup\n"
      "  -c file      Open command console on `file' (use `-' for std in/out)\n"
      "  -Z portnum   Use localhost:portnum for command console\n"
      "  -k portnum   Use localhost:portnum for serial I/O\n"
@@ -226,9 +242,10 @@ print_help(char *name)
      "                  uart=nr   number of uart (default=0)\n"
      "                  in=file   serial input will be read from file named `file'\n"
      "                  out=file  serial output will be written to `file'\n"
-     "                  port=nr   Use localhost:nr as server for serial line\n"
-     "                  iport=nr  Use localhost:nr as server for serial input\n"
-     "                  oport=nr  Use localhost:nr as server for serial output\n"
+     "                  port=nr   use localhost:nr as server for serial line\n"
+     "                  iport=nr  use localhost:nr as server for serial input\n"
+     "                  oport=nr  use localhost:nr as server for serial output\n"
+     "                  raw       perform non-interactive communication even on tty\n"
      "  -I options   `options' is a comma separated list of options according to\n"
      "               simulator interface. Known options are:\n"
      "                 if=memory[address]  turn on interface on given memory location\n"
@@ -236,11 +253,18 @@ print_help(char *name)
      "                 out=file            specify output file for IO\n"
      "  -p prompt    Specify string for prompt\n"
      "  -P           Prompt is a null ('\\0') character\n"
+     "  -o colors    `colors' is a list of color specification: what=colspec,...\n"
+     "               where colspec is : separated list of color options\n"
+     "               e.g.: prompt=b:white:black (bold white on black)\n"
+     "  -l           Use light theme (default is dark)\n"
+     "  -b           Black & white (non-color) theme\n"
+     "  -B           Beep on breakpoints\n"
      "  -g           Go, start simulation\n"
      "  -G           Go, start simulation, quit on stop\n"
      "  -a nr        Specify size of variable space (default=256)\n"
      "  -w           Writable flash\n"
      "  -V           Verbose mode\n"
+     "  -q           Quiet mode\n"
      "  -v           Print out version number and quit\n"
      "  -H           Print out types of known CPUs and quit\n"
      "  -h           Print out this help and quit\n"
@@ -254,7 +278,9 @@ enum {
   SOPT_USART,
   SOPT_PORT,
   SOPT_IPORT,
-  SOPT_OPORT
+  SOPT_OPORT,
+  SOPT_RAW,
+  SOPT_ERROR
 };
 
 static const char *S_opts[]= {
@@ -265,6 +291,7 @@ static const char *S_opts[]= {
   /*[SOPT_PORT]=*/	"port",
   /*[SOPT_IPORT]=*/	"iport",
   /*[SOPT_OPORT]=*/	"oport",
+  /*[SOPT_RAW]=*/	"raw",
   NULL
 };
 
@@ -287,10 +314,10 @@ cl_app::proc_arguments(int argc, char *argv[])
   int i, c;
   char opts[100], *cp, *subopts, *value;
   char *cpu_type= NULL;
-  bool /*s_done= DD_FALSE,*/ k_done= false;
+  bool /*s_done= false,*/ k_done= false;
   //bool S_i_done= false, S_o_done= false;
 
-  strcpy(opts, "c:C:p:PX:vVt:s:S:I:a:whHgGJ_");
+  strcpy(opts, "qc:C:e:p:PX:vVt:s:S:I:a:whHgGJo:blBR:_");
 #ifdef SOCKET_AVAIL
   strcat(opts, "Z:r:k:");
 #endif
@@ -302,12 +329,19 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  (strcmp(argv[i], "-args") == 0) ||
 	  (strcmp(argv[i], "-nx") == 0))
 	strcpy(argv[i], "-_");
+      if ((strcmp(argv[i], "-help")==0) ||
+	  (strcmp(argv[i], "--help")==0))
+	{
+	  print_help(get_name());
+	  exit(0);
+	}
     }
   
   while((c= getopt(argc, argv, opts)) != -1)
     switch (c)
       {
       case '_': break;
+      case 'q': quiet= true; break;
       case 'J': jaj= true; break;
       case 'g':
 	if (!options->set_value("go", this, true))
@@ -332,13 +366,20 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  fprintf(stderr, "Warning: No \"config_file\" option found to set "
 		  "parameter of -C as config file\n");
 	break;
+      case 'e':
+	startup_command+= optarg;
+	startup_command+= chars("\n");
+	break;
+      case 'R':
+        srnd(atoi(optarg));
+        break;
 #ifdef SOCKET_AVAIL
       case 'Z': case 'r':
 	{
 	  // By Sandeep
 	  // Modified by DD
 	  class cl_option *o;
-	  options->new_option(o= new cl_number_option(this, cchars("port_number"),
+	  options->new_option(o= new cl_number_option(this, "port_number",
 						      "Listen on port (-Z)"));
 	  o->init();
 	  o->hide();
@@ -348,12 +389,19 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  break;
 	}
 #endif
-      case 'p': {
-	if (!options->set_value("prompt", this, optarg))
-	  fprintf(stderr, "Warning: No \"prompt\" option found to set "
-		  "parameter of -p as default prompt\n");
-	break;
-      }
+      case 'p':
+	{
+	  if (!options->set_value("prompt", this, optarg))
+	    fprintf(stderr, "Warning: No \"prompt\" option found to set "
+		    "parameter of -p as default prompt\n");
+	  break;
+	}
+      case 'b':
+	{
+	  if (!options->set_value("black_and_white", this, bool(true)))
+	    fprintf(stderr, "Warning: No \"black_and_white\" option found to set\n");
+	  break;
+	}
       case 'P':
 	if (!options->set_value("null_prompt", this, bool(true)))
 	  fprintf(stderr, "Warning: No \"null_prompt\" option found\n");
@@ -369,7 +417,7 @@ cl_app::proc_arguments(int argc, char *argv[])
 	    XTAL*= 1e6;
 	  if (XTAL == 0)
 	    {
-	      fprintf(stderr, "Xtal frequency must be greather than 0\n");
+	      fprintf(stderr, "Xtal frequency must be greater than 0\n");
 	      exit(1);
 	    }
 	  if (!options->set_value("xtal", this, XTAL))
@@ -443,38 +491,52 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  if (!options->set_value("serial0_in_file", this, /*(void*)Ser_in*/s))
 	    fprintf(stderr, "Warning: No \"serial0_in_file\" option found to "
 		    "set parameter of -s as serial input file\n");
+	  free(s);
 	  s= format_string("\0010x%llx", (unsigned long long int)(fout));
 	  if (!options->set_value("serial0_out_file", this, /*Ser_out*/s))
 	    fprintf(stderr, "Warning: No \"serial0_out_file\" option found "
 		    "to set parameter of -s as serial output file\n");
+	  free(s);
 	  break;
 	}
 #endif
       case 'S':
 	{
 	  char *iname= NULL, *oname= NULL;
-	  int uart=0, port=0, iport= 0, oport= 0;
+	  int uart=0, port=0, iport= 0, oport= 0, so;
+	  bool ifirst= false, raw= false;
 	  subopts= optarg;
 	  while (*subopts != '\0')
 	    {
-	      switch (get_sub_opt(&subopts, S_opts, &value))
+	      so= get_sub_opt(&subopts, S_opts, &value);
+	      if (so != SOPT_RAW)
 		{
+		  if ((value == NULL) ||
+		      (*value == 0))
+		    so= SOPT_ERROR;
+		}
+	      switch (so)
+		{
+		case SOPT_ERROR:
+		  fprintf(stderr, "No value for -S suboption\n");
+		  exit(1);
+		  break;
+		case SOPT_RAW:
+		  raw= 1;
+		  break;
 		case SOPT_IN:
-		  if (value == NULL) {
-		    fprintf(stderr, "No value for -S in\n");
-		    exit(1);
-		  }
 		  iname= value;
+		  if (oname == NULL)
+		    ifirst= true;
 		  break;
 		case SOPT_OUT:
-		  if (value == NULL) {
-		    fprintf(stderr, "No value for -S out\n");
-		    exit(1);
-		  }
 		  oname= value;
 		  break;
 		case SOPT_UART: case SOPT_USART:
 		  uart= strtol(value, 0, 0);
+		  ifirst= false;
+		  iname= oname= NULL;
+		  port= iport= oport= 0;
 		  break;
 		case SOPT_PORT:
 		  port= strtol(value, 0, 0);
@@ -500,6 +562,30 @@ cl_app::proc_arguments(int argc, char *argv[])
 	    {
 	      char *s, *h;
 	      class cl_option *o;
+	      s= format_string("serial%d_raw", uart);
+	      if ((o= options->get_option(s)) == NULL)
+		{
+		  h= format_string("Use raw communication on uart%d files", uart);
+		  o= new cl_bool_option(this, s, h);
+		  o->init();
+		  o->hide();
+		  options->add(o);
+		  free(h);
+		}
+	      options->set_value(s, this, raw);
+	      free(s);
+	      s= format_string("serial%d_ifirst", uart);
+	      if ((o= options->get_option(s)) == NULL)
+		{
+		  h= format_string("Open input file for uart%d first", uart);
+		  o= new cl_bool_option(this, s, h);
+		  o->init();
+		  o->hide();
+		  options->add(o);
+		  free(h);
+		}
+	      options->set_value(s, this, ifirst);
+	      free(s);
 	      if (iname)
 		{
 		  s= format_string("serial%d_in_file", uart);
@@ -633,8 +719,52 @@ cl_app::proc_arguments(int argc, char *argv[])
 	    }
 	  break;
 	}
+      case 'o':
+	{
+	  const chars s= optarg;
+	  chars opt= s.token(",");
+	  while (opt.nempty())
+	    {
+	      chars col_name, col_value, opt_name;
+	      col_name= opt.token("=");
+	      col_value=opt.token("=");
+	      opt_name.format("color_%s", col_name.c_str());
+	      class cl_option *o= options->get_option(opt_name);
+	      if (o)
+		o->set_value(col_value);
+	      opt= s.token(",");
+	    }
+	  break;
+	}
+      case 'l':
+	set_option_s("color_prompt", "green:bwhite");
+	set_option_s("color_prompt_console", "blue:bwhite");
+	set_option_s("color_command", "blue:bwhite");
+	set_option_s("color_answer", "black:bwhite");
+	set_option_s("color_result", "bblue:bwhite");
+	set_option_s("color_dump_address", "blue:bwhite");
+	set_option_s("color_dump_number", "bblack:bwhite");
+	set_option_s("color_dump_char", "black:bwhite");
+	set_option_s("color_comment", "magenta:bwhite");
+	set_option_s("color_error", "red:bwhite");
+	set_option_s("color_ui_mkey", "green:bwhite");
+	set_option_s("color_ui_mitem", "bblack:bwhite");
+	set_option_s("color_ui_label", "black:bwhite");
+	set_option_s("color_ui_time", "blue:bwhite");
+	set_option_s("color_ui_title", "magenta:bwhite");
+	set_option_s("color_ui_run", "black:green");
+	set_option_s("color_ui_stop", "white:red");
+	set_option_s("color_ui_bit0", "white:black");
+	set_option_s("color_ui_bit1", "bred:black");
+	set_option_s("color_debug", "magenta:bwhite");
+	break;
+      case 'B':
+	if (!options->set_value("beep_break", this, (bool)true))
+	  fprintf(stderr, "Warning: No \"debug\" option found to set "
+		  "by -B parameter\n");	
+	break;
       case 'h':
-	print_help(cchars("s51"));
+	print_help(get_name());
 	exit(0);
 	break;
       case 'H':
@@ -692,11 +822,11 @@ cl_app::get_cmd(class cl_cmdline *cmdline)
   return(0);
 }
 */
-long
+t_mem
 cl_app::eval(chars expr)
 {
   expr_result= 0;
-  uc_yy_set_string_to_parse((char*)expr);
+  uc_yy_set_string_to_parse(expr);
   yyparse();
   uc_yy_free_string_to_parse();
   return expr_result;
@@ -705,15 +835,11 @@ cl_app::eval(chars expr)
 void
 cl_app::exec(chars line)
 {
-  class cl_console_base *c= commander->frozen_console;
-  if (c == NULL)
-    {
-      c= new cl_console_dummy();
-      c->init();
-    }
+  class cl_console_base *c= commander->frozen_or_actual();
   do
     {
-      class cl_cmdline *cmdline= new cl_cmdline(this, (char*)line, c);
+      c->un_redirect();
+      class cl_cmdline *cmdline= new cl_cmdline(this, line, c);
       cmdline->init();
       class cl_cmd *cm= commander->cmdset->get_cmd(cmdline, false/*c->is_interactive()*/);
       if (cm)
@@ -725,16 +851,47 @@ cl_app::exec(chars line)
 	  char *e= cmdline->cmd;
 	  if (strlen(e) > 0)
 	    {
-	      long l= eval(e);
-	      c->dd_printf("%ld\n", l);
+	      t_mem l= eval(e);
+	      c->dd_color("result");
+	      c->print_expr_result(l, NULL);
+	    }
+	}
+      line= cmdline->rest;
+      delete cmdline;
+    }
+  while (!line.empty());
+}
+
+void
+cl_app::exec(chars line, class cl_console_base *con)
+{
+  if (!con)
+    return;
+  do
+    {
+      con->un_redirect();
+      class cl_cmdline *cmdline= new cl_cmdline(this, line, con);
+      cmdline->init();
+      class cl_cmd *cm= commander->cmdset->get_cmd(cmdline, false/*c->is_interactive()*/);
+      if (cm)
+	{
+	  cm->work(this, cmdline, con);
+	}
+      else if (cmdline->get_name() != 0)
+	{
+	  char *e= cmdline->cmd;
+	  if (strlen(e) > 0)
+	    {
+	      t_mem l= eval(e);
+	      con->dd_color("result");
+	      con->print_expr_result(l, NULL);
+	      
 	    }
 	}
       line= cmdline->rest;
       delete cmdline;
     }
   while (!line.empty());
-  if (c != commander->frozen_console)
-    delete c;
 }
 
 /*
@@ -779,6 +936,10 @@ cl_app::build_cmdset(class cl_cmdset *cmdset)
   cmdset->add(cmd= new cl_super_cmd("conf", 0, cset));
   cmd->init();
   set_conf_help(cmd);
+
+  cmd= new cl_ver_cmd("version", 0);
+  cmdset->add(cmd);
+  cmd->init();
   
   cmd= new cl_help_cmd("help", 0);
   cmdset->add(cmd);
@@ -787,13 +948,21 @@ cl_app::build_cmdset(class cl_cmdset *cmdset)
 
   cmdset->add(cmd= new cl_quit_cmd("quit", 0));
   cmd->init();
-
+  cmd->add_name("exit");
+  
   cmdset->add(cmd= new cl_kill_cmd("kill", 0));
   cmd->init();
 
   cmdset->add(cmd= new cl_exec_cmd("exec", 0));
   cmd->init();
 
+  cmdset->add(cmd= new cl_echo_cmd("echo", 0));
+  cmd->init();
+  cmd->add_name("print");
+
+  cmdset->add(cmd= new cl_dev_cmd("dev", 0));
+  cmd->init();
+
   cmdset->add(cmd= new cl_expression_cmd("expression", 0));
   cmd->init();
   cmd->add_name("let");
@@ -837,8 +1006,8 @@ cl_app::build_cmdset(class cl_cmdset *cmdset)
     }
     cset->add(cmd= new cl_get_option_cmd("option", 0));
     cmd->init();
-    cset->add(cmd= new cl_show_error_cmd("error", 0));
-    cmd->init();
+    /*cset->add(cmd= new cl_show_error_cmd("error", 0));
+      cmd->init();*/
   }
   if (!super_cmd)
     {
@@ -880,6 +1049,11 @@ cl_app::mk_options(void)
   o->init();
   o->set_value((bool)false);
   
+  options->new_option(o= new cl_bool_option(this, "black_and_white",
+					    "Non-color console (-b)"));
+  o->init();
+  o->set_value((bool)false);
+  
   options->new_option(o= new cl_bool_option(this, "null_prompt",
 					    "Use \\0 as prompt (-P)"));
   o->init();
@@ -915,7 +1089,7 @@ cl_app::mk_options(void)
   options->new_option(o= new cl_float_option(this, "xtal",
 					     "Frequency of XTAL in Hz"));
   o->init();
-  o->set_value(11059200.0);
+  o->set_value(0.0);
 
   options->new_option(o= new cl_string_option(this, "cpu_type",
 					      "Type of controller (-t)"));
@@ -958,6 +1132,126 @@ cl_app::mk_options(void)
   options->new_option(o= new cl_bool_option(this, "echo_script",
 					    "Print breakpoint script before execute"));
   o->init();
+
+  options->new_option(o= new cl_bool_option(this, "beep_break",
+					    "Beep at breakpoint hit (-B)"));
+  o->init();
+  o->set_value((bool)false);
+
+  options->new_option(o= new cl_string_option(this, "expression_format",
+					      "Format specifier to print expression result"));
+  o->init();
+  o->set_value("d");
+  
+  options->new_option(o= new cl_string_option(this, "color_prompt",
+					      "Prompt color"));
+  o->init();
+  o->set_value("bwhite:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_prompt_console",
+					      "Color of console number in prompt"));
+  o->init();
+  o->set_value("yellow:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_command",
+					      "Color of entered command"));
+  o->init();
+  o->set_value("green:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_answer",
+					      "Answer color"));
+  o->init();
+  o->set_value("bwhite:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_result",
+					      "Result of expression"));
+  o->init();
+  o->set_value("byellow:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_dump_address",
+					      "Address color in dump"));
+  o->init();
+  o->set_value("byellow:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_dump_label",
+					      "Label color in dump"));
+  o->init();
+  o->set_value((char*)"bgreen:black");
+
+  options->new_option(o= new cl_string_option(this, "color_dump_number",
+					      "Value color in dump"));
+  o->init();
+  o->set_value("white:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_dump_char",
+					      "Text color in dump"));
+  o->init();
+  o->set_value("green:black");
+
+  options->new_option(o= new cl_string_option(this, "color_comment",
+					      "Comment color in disassembly"));
+  o->init();
+  o->set_value("magenta:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_error",
+					      "Text color in error messages"));
+  o->init();
+  o->set_value("red:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_debug",
+					      "Color of debug messages"));
+  o->init();
+  o->set_value("magenta:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_mkey",
+					      "Menu-key color on UI display"));
+  o->init();
+  o->set_value("b:yellow:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_mitem",
+					      "Menu-item color on UI display"));
+  o->init();
+  o->set_value("bwhite:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_label",
+					      "Label color on UI display"));
+  o->init();
+  o->set_value("white:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_time",
+					      "Color of time-value on UI display"));
+  o->init();
+  o->set_value("bblue:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_title",
+					      "Title color on UI display"));
+  o->init();
+  o->set_value("bmagenta:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_run",
+					      "Run state color on UI display"));
+  o->init();
+  o->set_value("black:green");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_bit0",
+					      "Bit 0 color on UI display"));
+  o->init();
+  o->set_value("white:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_bit1",
+					      "Bit 1 color on UI display"));
+  o->init();
+  o->set_value("bred:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_stop",
+					      "Stop state color on UI display"));
+  o->init();
+  o->set_value("white:red");
+  
+  options->new_option(o= new cl_number_option(this, "label_width",
+					      "Space to allow for labels in dumps and disassembly (-1 for auto)"));
+  o->init();
+  o->set_value((long)-1);
 }
 
 
@@ -975,6 +1269,20 @@ cl_app::dd_printf(const char *format, ...)
   return(i);
 }
 
+int
+cl_app::dd_cprintf(const char *color_name, const char *format, ...)
+{
+  va_list ap;
+
+  if (!commander)
+    return(0);
+
+  va_start(ap, format);
+  int i= commander->dd_cprintf(color_name, format, ap);
+  va_end(ap);
+  return(i);
+}
+
 int
 cl_app::debug(const char *format, ...)
 {
@@ -990,4 +1298,14 @@ cl_app::debug(const char *format, ...)
 }
 
 
+void
+cl_app::set_option_s(const char *opt_name, const char *new_value)
+{
+  class cl_option *o= options->get_option(opt_name);
+  if (o)
+    {
+      o->set_value(new_value);
+    }
+}
+
 /* End of app.cc */
