@@ -28,16 +28,17 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "ddconfig.h"
 
 #include <stdio.h>
-#include <sys/time.h>
+//#include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include HEADER_FD
-#include <errno.h>
+//#include <errno.h>
 #include <string.h>
 #if defined HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 # include <netinet/in.h>
 # include <arpa/inet.h>
+# include <sys/socket.h>
 #endif
 #include <stdlib.h>
 #include <ctype.h>
@@ -70,48 +71,44 @@ cl_history::cl_history(const char *aname):
 cl_ustrings(100, 10, aname)
 {
   nr= 0;
-  //actual_line= "";
 }
 
 cl_history::~cl_history(void)
 {
 }
 
-char *
+const char *
 cl_history::up(chars line)
 {
   replace(line);
   if (nr > 0)
     nr--;
-  return (char*)(Items[nr]);
+  if (nr < count)
+    return (char*)Items[nr];
+  return NULL;
 }
 
-char *
+const char *
 cl_history::down(chars line)
 {
   replace(line);
   if (nr < count)
     nr++;
   if (nr < count)
-    return (char*)(Items[nr]);
+    return (char*)Items[nr];
   return NULL;
 }
 
-char *
+void
 cl_history::enter(chars line)
 {
   if (count > 1000)
-    {
-      free_at(0);
-      /*if (nr > count)
-	nr= count;*/
-    }
+    free_at(0);
   if (!line.empty())
     {
       add(strdup(line));
       nr= count;
     }
-  return NULL;
 }
 
 void
@@ -140,6 +137,7 @@ cl_f::cl_f(void)
   server_port= -1;
   echo_of= NULL;
   echo_to= NULL;
+  echo_color= "";
   at_end= 0;
   last_used= first_free= 0;
   cooking= 0;
@@ -150,6 +148,7 @@ cl_f::cl_f(void)
   hist= new cl_history("history");
   proc_telnet= false;
   proc_escape= false;
+  type= F_UNKNOWN;
 }
 
 cl_f::cl_f(chars fn, chars mode):
@@ -163,6 +162,7 @@ cl_f::cl_f(chars fn, chars mode):
   server_port= -1;
   echo_of= NULL;
   echo_to= NULL;
+  echo_color= "";
   at_end= 0;
   last_used= first_free= 0;
   cooking= 0;
@@ -172,6 +172,8 @@ cl_f::cl_f(chars fn, chars mode):
   attributes_saved= 0;
   hist= new cl_history("history");
   proc_telnet= false;
+  proc_escape= false;
+  type= F_UNKNOWN;
 }
 
 cl_f::cl_f(int the_server_port)
@@ -184,6 +186,7 @@ cl_f::cl_f(int the_server_port)
   server_port= the_server_port;
   echo_of= NULL;
   echo_to= NULL;
+  echo_color= "";
   at_end= 0;
   last_used= first_free= 0;
   cooking= 0;
@@ -193,18 +196,20 @@ cl_f::cl_f(int the_server_port)
   attributes_saved= 0;
   hist= new cl_history("history");
   proc_telnet= false;
+  proc_escape= false;
+  type= F_UNKNOWN;
 }
 
 class cl_f *
 cl_f::copy(chars mode)
 {
-  class cl_f *io= mk_io(chars(""), chars(""));
+  class cl_f *io= mk_io("", "");
   io->use_opened(file_id, mode);
   return io;
 }
 
 static int
-open_flags(char *m)
+open_flags(const char *m)
 {
   if (strcmp(m, "r") == 0)
     return O_RDONLY;
@@ -227,6 +232,12 @@ cl_f::init(void)
   if (server_port > 0)
     {
       file_id= mk_srv_socket(server_port);
+      if (file_id <= 0)
+	{
+	  file_id= -1;
+	  own= false;
+	  return -1;
+	}
       listen(file_id, 50);
       own= true;
       tty= false;
@@ -235,7 +246,7 @@ cl_f::init(void)
   else if (!file_name.empty())
     {
       if (file_mode.empty())
-	file_mode= cchars("r+");
+	file_mode= "r+";
       if ((file_id= ::open(file_name, open_flags(file_mode), (S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH))) >= 0)
 	{
 	  tty= isatty(file_id);
@@ -253,13 +264,13 @@ cl_f::init(void)
 }
 
 int
-cl_f::use_opened(int opened_file_id, char *mode)
+cl_f::use_opened(int opened_file_id, const char *mode)
 {
   close();
   if (mode)
     file_mode= mode;
   else
-    file_mode= cchars("r+");
+    file_mode= "r+";
   own= false;
   if (opened_file_id >= 0)
     {
@@ -271,7 +282,7 @@ cl_f::use_opened(int opened_file_id, char *mode)
 }
 
 int
-cl_f::own_opened(int opened_file_id, char *mode)
+cl_f::own_opened(int opened_file_id, const char *mode)
 {
   use_opened(opened_file_id, mode);
   own= true;
@@ -279,7 +290,7 @@ cl_f::own_opened(int opened_file_id, char *mode)
 }
 
 int
-cl_f::use_opened(FILE *f, chars mode)
+cl_f::use_opened(FILE *f, const char *mode)
 {
   close();
   if (f)
@@ -298,7 +309,7 @@ cl_f::use_opened(FILE *f, chars mode)
 }
 
 int
-cl_f::own_opened(FILE *f, chars mode)
+cl_f::own_opened(FILE *f, const char *mode)
 {
   use_opened(f, mode);
   own= true;
@@ -306,7 +317,7 @@ cl_f::own_opened(FILE *f, chars mode)
 }
 
 int
-cl_f::open(char *fn)
+cl_f::open(const char *fn)
 {
   close();
   if (fn)
@@ -315,7 +326,7 @@ cl_f::open(char *fn)
 }
 
 int
-cl_f::open(char *fn, char *mode)
+cl_f::open(const char *fn, const char *mode)
 {
   close();
   if (mode)
@@ -695,7 +706,7 @@ cl_f::process(char c)
   // HISTORY
   else if (k == TU_UP)
     {
-      char *s= hist->up(line);
+      const char *s= hist->up(line);
       if (cursor > 0)
 	echo_cursor_go_left(cursor);
       echo_cursor_save();
@@ -712,7 +723,7 @@ cl_f::process(char c)
     }
   else if (k == TU_DOWN)
     {
-      char *s= hist->down(line);
+      const char *s= hist->down(line);
       if (cursor > 0)
 	echo_cursor_go_left(cursor);
       echo_cursor_save();
@@ -768,8 +779,7 @@ cl_f::process(char c)
 	  echo_cursor_restore();
 	}
     }
-  else if (//(k == 127) || /*DEL*/
-	   (k == TU_DEL))
+  else if (k == TU_DEL)
     {
       if (line[cursor] != 0)
 	{
@@ -844,7 +854,7 @@ cl_f::pick(void)
       at_end= 1;
     }
   if (i < 0)
-    ;
+    {}
   return i;
 }
 
@@ -961,7 +971,7 @@ cl_f::read_dev(int *buf, int max)
 
 
 int
-cl_f::write(char *buf, int count)
+cl_f::write(const char *buf, int count)
 {
   int i;
   if (file_id >= 0)
@@ -979,13 +989,13 @@ cl_f::write(char *buf, int count)
 	    {
 	      j= ::write(file_id, "\r\n", 2);
 	      if (j != 2)
-		;
+	        {}
 	    }
 	  else
 	    {
 	      j= ::write(file_id, &buf[i], 1);
 	      if (j != 1)
-		;
+	        {}
 	    }
 	}
       return i;
@@ -995,7 +1005,7 @@ cl_f::write(char *buf, int count)
 
 
 int
-cl_f::write_str(char *s)
+cl_f::write_str(const char *s)
 {
   if (!s ||
       !*s)
@@ -1004,15 +1014,6 @@ cl_f::write_str(char *s)
 }
 
 
-int
-cl_f::write_str(const char *s)
-{
-  if (!s ||
-      !*s)
-    return 0;
-  return write((char*)s, strlen((char*)s));
-}
-
 int
 cl_f::vprintf(const char *format, va_list ap)
 {
@@ -1057,7 +1058,7 @@ cl_f::echo_cursor_save()
 {
   if (echo_to)
     {
-      echo_to->write(cchars("\033[s"), 3);
+      echo_to->write("\033[s", 3);
       //echo_to->flush();
     }
 }
@@ -1067,7 +1068,7 @@ cl_f::echo_cursor_restore()
 {
   if (echo_to)
     {
-      echo_to->write(cchars("\033[u"), 3);
+      echo_to->write("\033[u", 3);
       //echo_to->flush();
     }
 }
@@ -1097,36 +1098,36 @@ cl_f::echo_cursor_go_right(int n)
 }
 
 void
-cl_f::echo_write(char *b, int l)
+cl_f::echo_write(const char *b, int l)
 {
   if (echo_to)
     {
+      if (echo_color.nempty())
+	echo_to->prntf("%s", echo_color.c_str());
       echo_to->write(b, l);
       //echo_to->flush();
     }
 }
 
-void
-cl_f::echo_write_str(char *s)
-{
-  if (echo_to)
-    {
-      echo_to->write_str(s);
-      //echo_to->flush();
-    }
-}
-
 void
 cl_f::echo_write_str(const char *s)
 {
   if (echo_to)
     {
+      if (echo_color.nempty())
+	echo_to->prntf("%s", echo_color.c_str());
       echo_to->write_str(s);
       //echo_to->flush();
     }
 }
 
-  
+void
+cl_f::set_echo_color(chars col)
+{
+  echo_color= col;
+}
+
+
 /* Device handling */
 
 void
@@ -1218,7 +1219,7 @@ cl_f::set_escape(bool val)
 }
 
 
-chars
+const char *
 fio_type_name(enum file_type t)
 {
   switch (t)
