@@ -25,46 +25,20 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA. */
 /*@1@*/
 
-#include "ddconfig.h"
+//#include "ddconfig.h"
 
 #include <stdio.h>
 
 #include "stypes.h"
 
-
 // No other global variable should exists!
 class cl_app *application;
+double app_start_at= 0;
 
 
 /* Delimiters to split up command line */
 char delimiters[]= " ,;\t\r\n";
 
-struct id_element mem_ids[]= {
-  { MEM_ROM  , "ROM  " },
-  { MEM_XRAM , "XRAM " },
-  { MEM_IRAM , "IRAM " },
-  { MEM_SFR  , "SFR  " },
-  { MEM_IXRAM, "IXRAM" },
-  { MEM_DUMMY, "DUMMY" },
-  { 0, 0 }
-};
-
-struct id_element mem_classes[]= {
-  { MEM_ROM  , "rom" },
-  { MEM_XRAM , "xram" },
-  { MEM_IRAM , "iram" },
-  { MEM_SFR  , "sfr" },
-  { MEM_IXRAM, "ixram" },
-  { MEM_DUMMY, "dummy" },
-  { 0, 0 }
-};
-
-struct id_element cpu_states[]= {
-  { stGO,	"OK" },
-  { stIDLE,	"Idle" },
-  { stPD,	"PowerDown" },
-  { 0, 0 }
-};
 
 struct id_element error_type_names[]= {
   { err_unknown,	"unclassified" },
@@ -282,7 +256,7 @@ const char *copying=
 "signed it.  However, nothing else grants you permission to modify or\n"
 "distribute the Program or its derivative works.  These actions are\n"
 "prohibited by law if you do not accept this License.  Therefore, by\n"
-"odifying or distributing the Program (or any work based on the\n"
+"modifying or distributing the Program (or any work based on the\n"
 "Program), you indicate your acceptance of this License to do so, and\n"
 "all its terms and conditions for copying, distributing or modifying\n"
 "the Program or works based on it.\n"
@@ -426,20 +400,6 @@ struct cpu_entry cpus_51[]=
     {NULL, CPU_NONE, 0, "", ""}
   };
 
-struct cpu_entry cpus_z80[]=
-  {
-    {"Z80"    , CPU_Z80,     0			, ""		, ""},
-
-    {"Z180"   , CPU_Z180,    0			, ""		, ""},
-    {"R2K"    , CPU_R2K,     0			, ""		, ""},
-    {"R3KA"   , CPU_R3KA,    0			, ""		, ""},
-    {"LR35902", CPU_LR35902, 0			, ""		, ""},
-
-    {"EZ80"   , CPU_EZ80,    0			, ""		, ""},
-    
-    {NULL, CPU_NONE, 0, "", ""}
-  };
-
 struct cpu_entry cpus_hc08[]=
   {
     {"HC08"   , CPU_HC08, 0			, ""		, ""},
@@ -551,7 +511,12 @@ struct cpu_entry cpus_stm8[]=
 
 struct cpu_entry cpus_pdk[]=
   {
-    {"PDK14"   , CPU_PDK14, 0			, ""		, ""},
+    {"PDK13"   , CPU_PDK13, 0			, "PDK13"		, ""},
+    {"PMC153"  , CPU_PDK13, 0			, "PDK13"		, ""},
+    {"PDK14"   , CPU_PDK14, 0			, "PDK14"		, ""},
+    {"PMS132B" , CPU_PDK14, 0			, "PDK14"		, ""},
+    {"PDK15"   , CPU_PDK15, 0			, "PDK15"		, ""},
+    {"PMS134"  , CPU_PDK15, 0			, "PDK15"		, ""},
     
     {NULL, CPU_NONE, 0, "", ""}
   };
