@@ -45,34 +45,16 @@ class cl_event_handler;
 // Cell flags
 enum cell_flag {
   CELL_NONE		= 0x00,
-  CELL_VAR		= 0x01, /* At least one variable points to it */
   CELL_INST		= 0x04,	/* Marked as instruction */
   CELL_FETCH_BRK	= 0x08,	/* Fetch breakpoint */
   CELL_READ_ONLY	= 0x10, /* Cell is readonly */
   CELL_NON_DECODED	= 0x40	/* Cell is not decoded (yet) */
 };
 
-enum dump_format {
-  // main formats
-  df_format	= 0x000f,
-  df_hex	= 0x0001,
-  df_string	= 0x0002,
-  df_ihex	= 0x0003,
-  df_binary	= 0x0004,
-  // modifiers
-  df_data_size	= 0x00f0,
-  df_1		= 0x0010,
-  df_2		= 0x0020,
-  df_4		= 0x0040,
-  df_8		= 0x0080,
-  // endianes
-  df_endian	= 0x0100,
-  df_little	= 0x0000,
-  df_big	= 0x0100,
-};
 
 #define CELL_GENERAL	(CELL_NORMAL|CELL_INST|CELL_FETCH_BRK)
 
+extern t_mem def_data;
 
 /*
  * 3rd version of memory system
@@ -80,6 +62,8 @@ enum dump_format {
 
 class cl_memory: public cl_base
 {
+private:
+  unsigned int uid;
 public:
   t_addr start_address;
 protected:
@@ -90,15 +74,19 @@ public:
   int width; // in bits
   t_mem data_mask;
   bool hidden;
+  chars altname;
 protected:
   t_addr dump_finished;
 public:
   cl_memory(const char *id, t_addr asize, int awidth);
   virtual ~cl_memory(void);
+  virtual bool is_named(const char *the_name) const;
+  virtual bool is_inamed(const char *the_name) const;
   virtual int init(void);
 
   t_addr get_start_address(void) { return(start_address); }
   t_addr get_size(void) { return(size); }
+  unsigned int get_uid() { return uid; }
   virtual void set_uc(class cl_uc *auc) { uc= auc; }
   virtual bool valid_address(t_addr addr);
   virtual t_addr inc_address(t_addr addr, int val);
@@ -107,18 +95,22 @@ public:
 
   virtual bool is_chip(void) { return(false); }
   virtual bool is_address_space(void) { return(false); }
-
+  
   virtual void err_inv_addr(t_addr addr);
   virtual void err_non_decoded(t_addr addr);
 
-  virtual t_addr dump(t_addr start, t_addr stop, int bpl, class cl_f *f);
-  virtual t_addr dump_s(t_addr start, t_addr stop, int bpl, class cl_f *f);
-  virtual t_addr dump_b(t_addr start, t_addr stop, int bpl, class cl_f *f);
-  virtual t_addr dump_i(t_addr start, t_addr stop, int bpl, class cl_f *f);
-  virtual t_addr dump(class cl_f *f);
-  virtual t_addr dump(enum dump_format fmt,
-		      t_addr start, t_addr stop, int bpl,
-		      class cl_f *f);
+  virtual t_addr dump(int smart, t_addr start, t_addr stop, int bitnr_high, int bitnr_low, int bpl, class cl_console_base *con);
+  virtual t_addr dump(int smart, t_addr start, t_addr stop, int bpl, class cl_console_base *con) {
+    return dump(smart, start, stop, -1, -1, bpl, con);
+  }
+  virtual t_addr dump(t_addr addr, int bitnr_high, int bitnr_low, class cl_console_base *con) {
+    return dump(2, addr, addr, bitnr_high, bitnr_low, -1, con);
+  }
+  virtual t_addr dump_s(t_addr start, t_addr stop, int bpl, class cl_console_base *con);
+  virtual t_addr dump_b(t_addr start, t_addr stop, int bpl, class cl_console_base *con);
+  virtual t_addr dump_i(t_addr start, t_addr stop, int bpl, class cl_console_base *con);
+  virtual t_addr dump(t_addr start, t_addr stop, int bpl, class cl_console_base *con) { return dump(1, start, stop, bpl, con); }
+  //virtual t_addr dump(class cl_console_base *con) { return(dump(df_smart, -1, -1, -1, con)); }
   virtual bool search_next(bool case_sensitive,
 			   t_mem *array, int len, t_addr *addr);
 
@@ -129,12 +121,12 @@ public:
   virtual t_mem read(t_addr addr)=0;
   virtual t_mem read(t_addr addr, enum hw_cath skip)=0;
   virtual t_mem get(t_addr addr)=0;
+  //virtual t_mem get8(t_addr addr)=0;
+  //virtual t_mem get16(t_addr addr)=0;
   virtual t_mem write(t_addr addr, t_mem val)=0;
   virtual void set(t_addr addr, t_mem val)=0;
-  virtual void set_bit1(t_addr addr, t_mem bits)=0;
-  virtual void set_bit0(t_addr addr, t_mem bits)=0;
 
-  virtual void print_info(chars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
 };
 
 
@@ -146,16 +138,11 @@ class cl_banker;
 
 class cl_memory_operator: public cl_base
 {
-protected:
-  //t_addr address;
+public:
   t_mem mask;
-  class cl_memory_operator *next_operator;
   class cl_memory_cell *cell;
 public:
-  cl_memory_operator(class cl_memory_cell *acell/*, t_addr addr*/);
-
-  virtual class cl_memory_operator *get_next(void) { return(next_operator); }
-  virtual void set_next(class cl_memory_operator *next) { next_operator= next;}
+  cl_memory_operator(class cl_memory_cell *acell);
 
   virtual bool match(class cl_hw *the_hw) { return(false); }
   virtual bool match(class cl_brk *brk) { return(false); }
@@ -172,7 +159,7 @@ class cl_bank_switcher_operator: public cl_memory_operator
  protected:
   class cl_banker *banker;
  public:
-  cl_bank_switcher_operator(class cl_memory_cell *acell/*, t_addr addr*/,
+  cl_bank_switcher_operator(class cl_memory_cell *acell,
 			    class cl_banker *the_banker);
   
   virtual t_mem write(t_mem val);
@@ -184,8 +171,8 @@ class cl_hw_operator: public cl_memory_operator
 protected:
   class cl_hw *hw;
 public:
-  cl_hw_operator(class cl_memory_cell *acell/*, t_addr addr*/,
-		 /*t_mem *data_place, t_mem the_mask,*/ class cl_hw *ahw);
+  cl_hw_operator(class cl_memory_cell *acell,
+		 class cl_hw *ahw);
 
   virtual bool match(class cl_hw *the_hw) { return(hw == the_hw); }
 
@@ -200,9 +187,9 @@ protected:
   class cl_uc *uc;
   class cl_brk *bp;
 public:
- cl_event_break_operator(class cl_memory_cell *acell/*, t_addr addr*/,
+  cl_event_break_operator(class cl_memory_cell *acell,
 			  class cl_uc *auc, class cl_brk *the_bp):
-  cl_memory_operator(acell/*, addr*/)
+  cl_memory_operator(acell)
   {
     uc= auc;
     bp= the_bp;
@@ -214,7 +201,7 @@ public:
 class cl_write_operator: public cl_event_break_operator
 {
 public:
-  cl_write_operator(class cl_memory_cell *acell/*, t_addr addr*/,
+  cl_write_operator(class cl_memory_cell *acell,
 		    class cl_uc *auc, class cl_brk *the_bp);
 
   virtual t_mem write(t_mem val);
@@ -223,7 +210,7 @@ public:
 class cl_read_operator: public cl_event_break_operator
 {
 public:
-  cl_read_operator(class cl_memory_cell *acell/*, t_addr addr*/,
+  cl_read_operator(class cl_memory_cell *acell,
 		   class cl_uc *auc, class cl_brk *the_bp);
 
   virtual t_mem read(void);
@@ -237,10 +224,10 @@ public:
 class cl_cell_data: public cl_abs_base
 {
  protected:
-  t_mem *data;
-  virtual t_mem d();
-  virtual void d(t_mem v);
-  virtual void dl(t_mem v);
+  /*t_mem*/void *data;
+  virtual t_mem d()=0;
+  virtual void d(t_mem v)=0;
+  virtual void dl(t_mem v) { d(v); };
 };
 
 class cl_memory_cell: public cl_cell_data
@@ -248,110 +235,163 @@ class cl_memory_cell: public cl_cell_data
 #ifdef STATISTIC
  public:
   unsigned long nuof_writes, nuof_reads;
+  class cl_memory *as;
 #endif
  public:
   t_mem mask;
-  t_mem def_data;
+  //t_mem def_data;
  protected:
   uchar width;
-  /*TYPE_UBYTE*/uchar flags;
-  class cl_memory_operator *operators;
+  uchar flags;
+  //class cl_memory_operator *operators;
+  class cl_memory_operator **ops;
  public:
+  cl_memory_cell();
   cl_memory_cell(uchar awidth);
   virtual ~cl_memory_cell(void);
   virtual int init(void);
-
-  virtual t_mem *get_data(void) { return(data); }
+  virtual void set_width(uchar awidth);
+  
+  virtual /*t_mem*/void *get_data(void) { return(data); }
   virtual t_mem get_mask(void) { return(mask); }
   virtual void set_mask(t_mem m) { mask= m; }
-  virtual /*TYPE_UBYTE*/uchar get_flags(void);
+  virtual uchar get_flags(void);
   virtual bool get_flag(enum cell_flag flag);
-  virtual void set_flags(/*TYPE_UBYTE*/uchar what);
+  virtual void set_flags(uchar what);
   virtual void set_flag(enum cell_flag flag, bool val);
   virtual uchar get_width(void) { return width; }
   
   virtual void un_decode(void);
   virtual void decode(class cl_memory_chip *chip, t_addr addr);
-  virtual void decode(t_mem *data_ptr);
-  virtual void decode(t_mem *data_ptr, t_mem bit_mask);
+  virtual void decode(void *data_ptr);
+  virtual void decode(void *data_ptr, t_mem bit_mask);
   
   virtual t_mem read(void);
+  virtual t_mem R(void) { return read(); }
   virtual t_mem read(enum hw_cath skip);
   virtual t_mem get(void);
   virtual t_mem write(t_mem val);
+  virtual t_mem W(t_mem val) { return write(val); }
   virtual t_mem set(t_mem val);
   virtual t_mem download(t_mem val);
-  
-  virtual t_mem add(long what);
-  virtual t_mem wadd(long what);
 
-  virtual void set_bit1(t_mem bits);
-  virtual void write_bit1(t_mem bits);
-  virtual void set_bit0(t_mem bits);
-  virtual void write_bit0(t_mem bits);
-  virtual void toggle_bits(t_mem bits);
-  virtual void wtoggle_bits(t_mem bits);
-  
+  virtual int nuof_ops(void);
   virtual void append_operator(class cl_memory_operator *op);
   virtual void prepend_operator(class cl_memory_operator *op);
+  virtual void remove_operator(class cl_memory_operator *op);
   virtual void del_operator(class cl_brk *brk);
   virtual void del_operator(class cl_hw *hw);
   virtual class cl_banker *get_banker(void);
+  virtual void set_brk(class cl_uc *uc, class cl_brk *brk);
+  virtual void del_brk(class cl_brk *brk);
   
   virtual class cl_memory_cell *add_hw(class cl_hw *hw/*, t_addr addr*/);
   virtual void remove_hw(class cl_hw *hw);
   virtual class cl_event_handler *get_event_handler(void);
 
-  virtual void print_info(chars pre, class cl_console_base *con);
-  virtual void print_operators(cchars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
+  virtual void print_operators(const char *pre, class cl_console_base *con);
 };
 
+/*
 class cl_bit_cell: public cl_memory_cell
 {
  public:
- cl_bit_cell(uchar awidth): cl_memory_cell(awidth) {}
+  cl_bit_cell(uchar awidth): cl_memory_cell(awidth) {}
   virtual t_mem d();
   virtual void d(t_mem v);
 };
+*/
+
+/* Cell for 1 byte storage */
 
 class cl_cell8: public cl_memory_cell
 {
  public:
- cl_cell8(uchar awidth): cl_memory_cell(awidth) {}
+  cl_cell8(uchar awidth): cl_memory_cell(awidth) {}
+  cl_cell8(): cl_memory_cell(8) {}
   virtual t_mem d();
   virtual void d(t_mem v);
+  virtual void dl(t_mem v);
 };
 
 class cl_bit_cell8: public cl_memory_cell
 {
  public:
- cl_bit_cell8(uchar awidth): cl_memory_cell(awidth) {}
+  cl_bit_cell8(uchar awidth): cl_memory_cell(awidth) {}
+  cl_bit_cell8(): cl_memory_cell(8) {}
   virtual t_mem d();
   virtual void d(t_mem v);
 };
 
+
+/* Cell for 2 byte storage */
+
 class cl_cell16: public cl_memory_cell
 {
  public:
- cl_cell16(uchar awidth): cl_memory_cell(awidth) {}
+  cl_cell16(uchar awidth): cl_memory_cell(awidth) {}
+  cl_cell16(): cl_memory_cell(16) {}
   virtual t_mem d();
   virtual void d(t_mem v);
+  virtual void dl(t_mem v);
 };
 
 class cl_bit_cell16: public cl_memory_cell
 {
  public:
- cl_bit_cell16(uchar awidth): cl_memory_cell(awidth) {}
+  cl_bit_cell16(uchar awidth): cl_memory_cell(awidth) {}
   virtual t_mem d();
   virtual void d(t_mem v);
 };
 
 
+/* Cell for 4 byte storage */
+
+class cl_cell32: public cl_memory_cell
+{
+ public:
+  cl_cell32(uchar awidth): cl_memory_cell(awidth) {}
+  cl_cell32(): cl_memory_cell(32) {}
+  virtual t_mem d();
+  virtual void d(t_mem v);
+  virtual void dl(t_mem v);
+};
+
+class cl_mc32: public cl_cell32
+{
+public:
+  cl_mc32(uchar awidth): cl_cell32(awidth)
+  {
+    data= &def_data;
+    flags&= ~CELL_NON_DECODED;
+  }
+  cl_mc32(): cl_cell32(32)
+  {
+    data= &def_data;
+    flags&= ~CELL_NON_DECODED;
+  }
+};
+
+class cl_bit_cell32: public cl_memory_cell
+{
+ public:
+  cl_bit_cell32(uchar awidth): cl_memory_cell(awidth) {}
+  virtual t_mem d();
+  virtual void d(t_mem v);
+};
+
+
+/* Dummy cell */
+
 class cl_dummy_cell: public cl_memory_cell
 {
 public:
+  cl_dummy_cell(): cl_memory_cell() {}
   cl_dummy_cell(uchar awidth): cl_memory_cell(awidth) {}
 
+  virtual t_mem d();
+  virtual void d(t_mem v);
   virtual t_mem write(t_mem val);
   virtual t_mem set(t_mem val);
 };
@@ -374,20 +414,20 @@ class cl_address_space: public cl_memory
  public:
   cl_address_space(const char *id, t_addr astart, t_addr asize, int awidth);
   virtual ~cl_address_space(void);
-
+  virtual int init(void);
+  virtual class cl_memory_cell *cell_template();
+  
   virtual bool is_address_space(void) { return(true); }
 
   virtual t_mem read(t_addr addr);
   virtual t_mem read(t_addr addr, enum hw_cath skip);
   virtual t_mem get(t_addr addr);
+  //virtual t_mem get8(t_addr addr) { return get(addr)&0xff; }
+  //virtual t_mem get16(t_addr addr) { return get(addr)&0xffff; }
   virtual t_mem write(t_addr addr, t_mem val);
   virtual void set(t_addr addr, t_mem val);
   virtual void download(t_addr, t_mem val);
   
-  virtual t_mem wadd(t_addr addr, long what);
-  virtual void set_bit1(t_addr addr, t_mem bits);
-  virtual void set_bit0(t_addr addr, t_mem bits);
-  
   virtual class cl_memory_cell *get_cell(t_addr addr);
   virtual int get_cell_flag(t_addr addr);
   virtual bool get_cell_flag(t_addr addr, enum cell_flag flag);
@@ -418,16 +458,16 @@ class cl_address_space: public cl_memory
   virtual void set_nuof_writes(unsigned long value) {}
 #endif
 
-  virtual void print_info(chars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
 };
 
-class cl_address_space_list: public cl_list
+class cl_memory_list: public cl_list
 {
 protected:
   class cl_uc *uc;
 public:
-  cl_address_space_list(class cl_uc *the_uc);
-  virtual t_index add(class cl_address_space *mem);
+  cl_memory_list(class cl_uc *the_uc, const char *name);
+  virtual t_index add(class cl_memory *mem);
 };
 
 
@@ -435,35 +475,78 @@ public:
  * Memory chip (storage)
  */
 
-class cl_memory_chip: public cl_memory
+class cl_chip_data: public cl_memory
+{
+public:
+  cl_chip_data(const char *id, t_addr asize, int awidth);
+  virtual t_mem d(t_addr addr)= 0;
+  virtual void d(t_addr addr, t_mem v)= 0;
+};
+
+class cl_memory_chip: public cl_chip_data
 {
 protected:
-  t_mem *array;
+  void *array;
   int init_value;
   bool array_is_mine;
+  int alloc_size;
+  int bwidth; // size of array elements, in bytes
 public:
   cl_memory_chip(const char *id, int asize, int awidth, int initial= -1);
-  cl_memory_chip(const char *id, int asize, int awidth, t_mem *aarray);
+  cl_memory_chip(const char *id, int asize, int awidth, void *aarray, int arrsize);
   virtual ~cl_memory_chip(void);
   virtual int init(void);
 
   virtual bool is_chip(void) { return(true); }
 
-  virtual t_mem *get_slot(t_addr addr);
-  virtual t_addr is_slot(t_mem *data_ptr);
+  virtual void *get_slot(t_addr addr);
+  virtual bool is_slot(void *data_ptr, t_addr *addr_of);
   
-  virtual t_mem read(t_addr addr) { return(get(addr)); }
-  virtual t_mem read(t_addr addr, enum hw_cath skip) { return(get(addr)); }
-  virtual t_mem get(t_addr addr);
-  virtual t_mem write(t_addr addr, t_mem val) { set(addr, val); return(val); }
-  virtual void set(t_addr addr, t_mem val);
-  virtual void set_bit1(t_addr addr, t_mem bits);
-  virtual void set_bit0(t_addr addr, t_mem bits);
+  virtual t_mem read(t_addr addr) { return d(addr); }
+  virtual t_mem read(t_addr addr, enum hw_cath skip) { return d(addr); }
+  virtual t_mem get(t_addr addr) { return d(addr); }
+  virtual t_mem write(t_addr addr, t_mem val) { d(addr, val); return(val); }
+  virtual void set(t_addr addr, t_mem val) { d(addr, val); }
 
-  virtual void print_info(chars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
 };
 
-  
+class cl_chip8: public cl_memory_chip
+{
+public:
+  cl_chip8(const char *id, int asize, int awidth, int initial= -1);
+  cl_chip8(const char *id, int asize, int awidth, void *aarray, int arrsize);
+public:
+  virtual t_mem d(t_addr addr);
+  virtual void d(t_addr addr, t_mem v);
+};
+
+class cl_chip16: public cl_memory_chip
+{
+public:
+  cl_chip16(const char *id, int asize, int awidth, int initial= -1);
+  cl_chip16(const char *id, int asize, int awidth, void *aarray, int arrsize);
+public:
+  virtual t_mem d(t_addr addr);
+  virtual void d(t_addr addr, t_mem v);
+};
+
+class cl_chip32: public cl_memory_chip
+{
+public:
+  cl_chip32(const char *id, int asize, int awidth, int initial= -1);
+  cl_chip32(const char *id, int asize, int awidth, void *aarray, int arrsize);
+public:
+  virtual t_mem d(t_addr addr);
+  virtual void d(t_addr addr, t_mem v);
+};
+
+extern class cl_memory_chip *new_chip(const char *id,
+				      int asize,
+				      int awidth,
+				      int initial= -1);
+
+
 /*
  * Address decoder
  */
@@ -483,9 +566,13 @@ public:
   virtual int init(void);
   virtual bool is_banker() { return false; }
   virtual bool is_bander() { return false; }
+  virtual bool uses_chip(class cl_memory *chip) { return chip==memchip; }
 
   virtual bool activate(class cl_console_base *con);
 
+  t_addr as_to_chip(t_addr addr) { return addr - as_begin + chip_begin; }
+  t_addr chip_to_as(t_addr addr) { return addr - chip_begin + as_begin; }
+
   virtual bool fully_covered_by(t_addr begin, t_addr end);
   virtual bool is_in(t_addr begin, t_addr end);
   virtual bool covers(t_addr begin, t_addr end);
@@ -493,7 +580,7 @@ public:
   virtual bool shrink_out_of(t_addr begin, t_addr end);
   virtual class cl_address_decoder *split(t_addr begin, t_addr end);
 
-  virtual void print_info(chars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
 };
 
 
@@ -513,6 +600,7 @@ class cl_banker: public cl_address_decoder
   int bank;
   class cl_address_decoder **banks;
   int shift_by, shift2_by;
+  class cl_memory_operator *op1, *op2;
  public:
   cl_banker(class cl_address_space *the_banker_as,
 	    t_addr the_banker_addr,
@@ -541,8 +629,9 @@ class cl_banker: public cl_address_decoder
   virtual t_mem actual_bank();
   virtual bool activate(class cl_console_base *con);
   virtual bool switch_to(int bank_nr, class cl_console_base *con);
+  virtual bool uses_chip(class cl_memory *chip);
   
-  virtual void print_info(chars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
 };
 
 
@@ -567,7 +656,7 @@ class cl_bander: public cl_address_decoder
   virtual bool is_bander() { return true; }
 
   virtual bool activate(class cl_console_base *con);
-  virtual void print_info(chars pre, class cl_console_base *con);
+  virtual void print_info(const char *pre, class cl_console_base *con);
 };
 
 
@@ -580,8 +669,8 @@ protected:
 public:
   cl_decoder_list(t_index alimit, t_index adelta, bool bychip);
 
-  virtual void *key_of(void *item);
-  virtual int compare(void *key1, void *key2);
+  virtual const void *key_of(const void *item) const;
+  virtual int compare(const void *key1, const void *key2);
 };
 
 
