@@ -44,17 +44,6 @@
 
 #include "version.h"
 
-/** C99 isblank implementation for Visual C++ 2010
-*/
-
-#if defined(_MSC_VER) && (_MSC_VER < 1800)
-int
-isblank(int c)
-{
-  return (c == ' ' || c == '\t');
-}
-#endif
-
 /** Given an array of name, value string pairs creates a new hash
     containing all of the pairs.
 */
@@ -107,7 +96,7 @@ shell_escape (const char *str)
               if (str > begin)
                 dbuf_append (&dbuf, begin, str - begin);
 
-              /* append additional beckslash */
+              /* append additional backslash */
               ++backshl;
 
               /* special handling if last chars before double quote are backslashes */
@@ -128,7 +117,7 @@ shell_escape (const char *str)
             }
           else if ('%' == *str)
             {
-              /* diseble env. variable expansion */
+              /* disable env. variable expansion */
               /* append the remaining characters */
               if (begin && str > begin)
                 dbuf_append (&dbuf, begin, str - begin);
@@ -244,6 +233,42 @@ shell_escape (const char *str)
 #endif
 }
 
+
+/** Escape string for string constants.
+    Returns dynamically allocated string, which should be free-ed.
+    TODO: Maybe handle other non-printable characters.
+*/
+char *
+string_escape (const char *str)
+{
+  struct dbuf_s dbuf;
+
+  dbuf_init (&dbuf, 128);
+
+  while (*str)
+    {
+      switch (*str)
+        {
+        case '\\': case '"':
+          dbuf_append_char (&dbuf, '\\');
+          dbuf_append_char (&dbuf, *str);
+          break;
+
+        case '\n':
+          dbuf_append_str (&dbuf, "\\n");
+          break;
+
+        default:
+          dbuf_append_char (&dbuf, *str);
+          break;
+        }
+      ++str;
+    }
+
+  return dbuf_detach_c_str (&dbuf);
+}
+
+
 /** Prints elements of the set to the file, each element on new line
 */
 void
@@ -294,7 +319,7 @@ processStrSet (set * list, const char *pre, const char *post, char *(*func)(cons
   return new_list;
 }
 
-/** Given a set returns a string containing all of the strings seperated
+/** Given a set returns a string containing all of the strings separated
     by spaces. The returned string is on the heap.
 */
 const char *
@@ -547,7 +572,7 @@ buildCmdLine (const char **cmds, const char *p1, const char *p2, const char *p3,
                         if (*tmp != '\0')
                           {
                             if (sep)
-                              dbuf_append_char (&dbuf, ' ');    /* seperate it */
+                              dbuf_append_char (&dbuf, ' ');    /* separate it */
                             dbuf_append_str (&dbuf, tmp);
                             tmp++;
                             sep = 1;
@@ -566,7 +591,7 @@ buildCmdLine (const char **cmds, const char *p1, const char *p2, const char *p3,
           if (par && *par != '\0')
             {
               if (!first && sep)
-                dbuf_append_char (&dbuf, ' ');  /* seperate it */
+                dbuf_append_char (&dbuf, ' ');  /* separate it */
               dbuf_append_str (&dbuf, par);
               sep = 0;
             }
@@ -576,7 +601,7 @@ buildCmdLine (const char **cmds, const char *p1, const char *p2, const char *p3,
       if (*from != '\0')
         {
           if (!first && sep)
-            dbuf_append_char (&dbuf, ' ');      /* seperate it */
+            dbuf_append_char (&dbuf, ' ');      /* separate it */
           dbuf_append_str (&dbuf, from);
           sep = 0;
         }
@@ -736,7 +761,6 @@ getBuildEnvironment (void)
 #endif
 }
 
-#if defined(HAVE_VSNPRINTF) || defined(HAVE_VSPRINTF)
 size_t
 SDCCsnprintf (char *dst, size_t n, const char *fmt, ...)
 {
@@ -745,12 +769,7 @@ SDCCsnprintf (char *dst, size_t n, const char *fmt, ...)
 
   va_start (args, fmt);
 
-#if defined(HAVE_VSNPRINTF)
   len = vsnprintf (dst, n, fmt, args);
-#else
-  vsprintf (dst, fmt, args);
-  len = strlen (dst) + 1;
-#endif
 
   va_end (args);
 
@@ -765,7 +784,6 @@ SDCCsnprintf (char *dst, size_t n, const char *fmt, ...)
 
   return len;
 }
-#endif
 
 /** Pragma tokenizer
  */
@@ -857,8 +875,8 @@ hexEscape (const char **src)
 
 /*------------------------------------------------------------------*/
 /* universalEscape - process an hex constant of exactly four digits */
-/* return the hex value, throw a warning for illegal octal          */
-/* adjust src to point at the last proccesed char                   */
+/* return the hex value, throw an error warning for invalid hex     */
+/* adjust src to point at the last processed char                   */
 /*------------------------------------------------------------------*/
 
 unsigned long int
@@ -877,22 +895,23 @@ universalEscape (const char **str, unsigned int n)
 
   for (digits = 0; digits < n; ++digits)
     {
-      if (**str >= '0' && **str <= '7')
+      if (**str >= '0' && **str <= '9')
         {
           value = (value << 4) + (**str - '0');
           ++*str;
         }
-      else if ((**str | 0x20) >= 'a' && (**str | 0x20) <= 'f')
+      else if (tolower((unsigned char)(**str)) >= 'a' && (tolower((unsigned char)(**str)) <= 'f'))
         {
-          value = (value << 4) + (**str - 'a' + 10);
+          value = (value << 4) + (tolower((unsigned char)(**str)) - 'a' + 10);
           ++*str;
         }
       else
           break;
     }
-  if (digits != n || value < 0x00a0 && value != 0x0024 && value != 0x0040 && value != 0x0060 || value >= 0xd800 && 0xdfff >= value)
+  if (digits != n || value < 0x00a0 && value != 0x0024 && value != 0x0040 && value != 0x0060 || value >= 0xd800 && 0xdfff >= value ||
+    value > 0x10ffff) // Additional diagnostic required in C23, but since it is just a warning, we enable it even for older standards.
     {
-      werror (E_INVALID_UNIVERSAL, s);
+      werror (W_INVALID_UNIVERSAL, s);
     }
 
   return value;
@@ -901,7 +920,7 @@ universalEscape (const char **str, unsigned int n)
 /*------------------------------------------------------------------*/
 /* octalEscape - process an octal constant of max three digits      */
 /* return the octal value, throw a warning for illegal octal        */
-/* adjust src to point at the last proccesed char                   */
+/* adjust src to point at the last processed char                   */
 /*------------------------------------------------------------------*/
 
 unsigned long int
@@ -932,16 +951,16 @@ octalEscape (const char **str)
   Copies source string to a dynamically allocated buffer interpreting
   escape sequences and special characters
 
-  /param src  Buffer containing the source string with escape sequecnes
-  /param size Pointer to loction where the resulting buffer length is written
+  /param src  Buffer containing the source string with escape sequences
+  /param size Pointer to location where the resulting buffer length is written
   /return Dynamically allocated resulting buffer
 */
 
 const char *
 copyStr (const char *src, size_t *size)
 {
- const char *begin = NULL;
- struct dbuf_s dbuf;
+  const char *begin = NULL;
+  struct dbuf_s dbuf;
 
   dbuf_init(&dbuf, 128);
 
@@ -1040,7 +1059,7 @@ copyStr (const char *src, size_t *size)
             {
               char s[5] = "\0\0\0\0";
               if (c < 0x80)
-                s[0] = c;
+                s[0] = (char)c;
               else if (c < 0x800)
                 {
                   s[0] = (c >> 6) & 0x1f | 0xc0;
@@ -1064,7 +1083,7 @@ copyStr (const char *src, size_t *size)
               dbuf_append_str (&dbuf, s);
             }
           else
-            dbuf_append_char (&dbuf, c);
+            dbuf_append_char (&dbuf, (char)c);
 
           ++src;
         }
