@@ -42,8 +42,6 @@ static OPTION _ds390_options[] =
     { 0, OPTION_FLAT24_MODEL,   NULL, "use the flat24 model for the ds390 (default)" },
     { 0, OPTION_STACK_8BIT,     NULL, "use the 8bit stack for the ds390 (not supported yet)" },
     { 0, OPTION_STACK_SIZE,     &options.stack_size, "Tells the linker to allocate this space for stack", CLAT_INTEGER },
-    { 0, "--pack-iram",         NULL, "Tells the linker to pack variables in internal ram (default)"},
-    { 0, "--no-pack-iram",      &options.no_pack_iram, "Deprecated: Tells the linker not to pack variables in internal ram"},
     { 0, "--stack-10bit",       &options.stack10bit, "use the 10bit stack for ds390 (default)" },
     { 0, "--use-accelerator",   &options.useAccelerator, "generate code for ds390 arithmetic accelerator"},
     { 0, "--protect-sp-update", &options.protect_sp_update, "will disable interrupts during ESP:SP updates"},
@@ -91,7 +89,7 @@ static builtins __ds390_builtins[] = {
     { "__builtin_inp","v",3,{"cx*","cx*","i"}},        /* void __builtin_inp        (xdata char *,xdata char *,int) */
     /* __builtin_inp - used to write to a memory mapped port, increment first pointer */
     { "__builtin_outp","v",3,{"cx*","cx*","i"}},       /* void __builtin_outp       (xdata char *,xdata char *,int) */
-    { "__builtin_swapw","us",1,{"us"}},                /* unsigned short __builtin_swapw (unsigned short) */
+    { "__builtin_swapw","Us",1,{"Us"}},                /* unsigned short __builtin_swapw (unsigned short) */
     { "__builtin_memcmp_x2x","c",3,{"cx*","cx*","i"}}, /* void __builtin_memcmp_x2x (xdata char *,xdata char *,int) */
     { "__builtin_memcmp_c2x","c",3,{"cx*","cp*","i"}}, /* void __builtin_memcmp_c2x (xdata char *,code  char *,int) */
     { NULL , NULL,0, {NULL}}                           /* mark end of table */
@@ -99,6 +97,7 @@ static builtins __ds390_builtins[] = {
 void ds390_assignRegisters (ebbIndex * ebbi);
 
 static int regParmFlg = 0;      /* determine if we can register a parameter */
+static struct sym_link *regParmFuncType;
 
 static void
 _ds390_init (void)
@@ -110,11 +109,18 @@ static void
 _ds390_reset_regparm (struct sym_link *funcType)
 {
   regParmFlg = 0;
+  regParmFuncType = funcType;
 }
 
 static int
 _ds390_regparm (sym_link * l, bool reentrant)
 {
+  if (IFFUNC_HASVARARGS (regParmFuncType))
+    return 0;
+
+  if (IS_STRUCT (l))
+    return 0;
+
     if (IS_SPEC(l) && (SPEC_NOUN(l) == V_BIT))
         return 0;
     if (options.parms_in_bank1 == 0) {
@@ -191,8 +197,9 @@ _ds390_finaliseOptions (void)
     }
   else
     {
-      port->s.fptr_size = 3;
-      port->s.gptr_size = 4;
+      port->s.far_ptr_size = 3;
+      port->s.funcptr_size = 3;
+      port->s.ptr_size = 4;
 
       port->stack.isr_overhead += 2;      /* Will save dpx on ISR entry. */
 
@@ -215,12 +222,6 @@ _ds390_finaliseOptions (void)
           if (!options.stack_loc) options.stack_loc = 0x400008;
         }
 
-      /* generate native code 16*16 mul/div */
-      if (options.useAccelerator)
-        port->support.muldiv=2;
-      else
-        port->support.muldiv=1;
-
       /* Fixup the memory map for the stack; it is now in
        * far space and requires an FPOINTER to access it.
        */
@@ -379,7 +380,7 @@ _ds390_genInitStartup (FILE *of)
       fprintf (of, "\tmov\tsp,#__start__stack - 1\n");     /* MOF */
     }
 
-  fprintf (of, "\tlcall\t__sdcc_external_startup\n");
+  fprintf (of, "\tlcall\t___sdcc_external_startup\n");
   fprintf (of, "\tmov\ta,dpl\n");
   fprintf (of, "\tjz\t__sdcc_init_data\n");
   fprintf (of, "\tljmp\t__sdcc_program_startup\n");
@@ -475,7 +476,12 @@ static bool cseCostEstimation (iCode *ic, iCode *pdic)
 
 bool _ds390_nativeMulCheck(iCode *ic, sym_link *left, sym_link *right)
 {
-    return FALSE; // #STUB
+  if (IS_BITINT (OP_SYM_TYPE (IC_RESULT(ic))) && SPEC_BITINTWIDTH (OP_SYM_TYPE (IC_RESULT(ic))) % 8)
+    return false;
+
+  return
+    getSize (left) == 1 && getSize (right) == 1 ||
+    options.useAccelerator && getSize (left) == 2 && getSize (right) == 2;
 }
 
 /* Indicate which extended bit operations this port supports */
@@ -484,7 +490,7 @@ hasExtBitOp (int op, int size)
 {
   if (op == RRC
       || op == RLC
-      || op == GETHBIT
+      || op == GETABIT
       || (op == SWAP && size <= 2)
      )
     return TRUE;
@@ -1029,9 +1035,11 @@ PORT ds390_port =
     NULL,
     NULL,
     NULL,
+    NULL,
+    NULL,
   },
-  /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-  { 1, 2, 2, 4, 8, 1, 2, 3, 1, 4, 4 },
+  /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float, _BitInt (in bits) */
+  { 1, 2, 2, 4, 8, 1, 2, 3, 2, 3, 1, 4, 64 },
 
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },           /* far, near, xstack, code */
@@ -1061,12 +1069,14 @@ PORT ds390_port =
     NULL,
     NULL,
     1,
+    true,                       // unqualified pointer can point to __sfr: TODO: CHECK IF THIS IS ACTUALLY SUPPORTED. Set to true to emulate behaviour of rpevious version of sdcc for now.
     1                           // No fancy alignments supported.
   },
   { NULL, NULL },
-  { +1, 1, 4, 1, 1, 0 },
+  0,                            // ABI revision
+  { +1, 1, 4, 1, 1, 0, 0 },
   /* ds390 has an 16 bit mul & div */
-  { 2, -1 },
+  { -1, FALSE },
   { ds390_emitDebuggerSymbol },
   {
     255/4,      /* maxCount */
@@ -1129,8 +1139,6 @@ static OPTION _tininative_options[] =
     { 0, OPTION_FLAT24_MODEL,   NULL, "use the flat24 model for the ds390 (default)" },
     { 0, OPTION_STACK_8BIT,     NULL, "use the 8bit stack for the ds390 (not supported yet)" },
     { 0, OPTION_STACK_SIZE,     &options.stack_size, "Tells the linker to allocate this space for stack", CLAT_INTEGER },
-    { 0, "--pack-iram",         NULL, "Tells the linker to pack variables in internal ram (default)"},
-    { 0, "--no-pack-iram",      &options.no_pack_iram, "Deprecated: Tells the linker not to pack variables in internal ram"},
     { 0, "--stack-10bit",       &options.stack10bit, "use the 10bit stack for ds390 (default)" },
     { 0, "--use-accelerator",   &options.useAccelerator, "generate code for ds390 arithmetic accelerator"},
     { 0, "--protect-sp-update", &options.protect_sp_update, "will disable interrupts during ESP:SP updates"},
@@ -1160,13 +1168,14 @@ static void _tininative_finaliseOptions (void)
         options.model = MODEL_FLAT24 ;
         fprintf(stderr,"TININative supports only MODEL FLAT24\n");
     }
-    port->s.fptr_size = 3;
-    port->s.gptr_size = 4;
+    port->s.far_ptr_size = 3;
+    port->s.funcptr_size = 3;
+    port->s.ptr_size = 4;
 
     port->stack.isr_overhead += 2;      /* Will save dpx on ISR entry. */
 
     port->stack.call_overhead += 2;     /* This acounts for the extra byte
-                                         * of return addres on the stack.
+                                         * of return address on the stack.
                                          * but is ugly. There must be a
                                          * better way.
                                          */
@@ -1181,12 +1190,6 @@ static void _tininative_finaliseOptions (void)
 
     if (!options.stack_loc) options.stack_loc = 0x400008;
 
-    /* generate native code 16*16 mul/div */
-    if (options.useAccelerator)
-        port->support.muldiv=2;
-    else
-        port->support.muldiv=1;
-
     /* Fixup the memory map for the stack; it is now in
      * far space and requires a FPOINTER to access it.
      */
@@ -1374,9 +1377,10 @@ PORT tininative_port =
     NULL,
     NULL,
     NULL,
+    NULL,
   },
-  /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-  { 1, 2, 2, 4, 8, 1, 3, 3, 1, 4, 4 },
+  /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float, _BitInt (in bits) */
+  { 1, 2, 2, 4, 8, 1, 3, 3, 3, 3, 1, 4, 64 },
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },           /* far, near, xstack, code */
 
@@ -1405,12 +1409,14 @@ PORT tininative_port =
     NULL,
     NULL,
     1,
+    true,                       // unqualified pointer can point to __sfr: TODO: CHECK IF THIS IS ACTUALLY SUPPORTED. Set to true to emulate behaviour of rpevious version of sdcc for now.
     1                           // No fancy alignments supported.
   },
   { NULL, NULL },
-  { +1, 1, 4, 1, 1, 0 },
+  0,                            // ABI revision
+  { +1, 1, 4, 1, 1, 0, 0 },
   /* ds390 has an 16 bit mul & div */
-  { 2, -1 },
+  { -1, FALSE },
   { ds390_emitDebuggerSymbol },
   {
     255/4,      /* maxCount */
@@ -1441,7 +1447,7 @@ PORT tininative_port =
   _ds390_regparm,
   NULL,
   NULL,
-  NULL,
+  _ds390_nativeMulCheck,
   hasExtBitOp,                  /* hasExtBitOp */
   oclsExpense,                  /* oclsExpense */
   FALSE,
@@ -1489,8 +1495,6 @@ static OPTION _ds400_options[] =
     { 0, OPTION_FLAT24_MODEL,   NULL, "use the flat24 model for the ds400 (default)" },
     { 0, OPTION_STACK_8BIT,     NULL, "use the 8bit stack for the ds400 (not supported yet)" },
     { 0, OPTION_STACK_SIZE,     &options.stack_size, "Tells the linker to allocate this space for stack", CLAT_INTEGER },
-    { 0, "--pack-iram",         NULL, "Tells the linker to pack variables in internal ram (default)"},
-    { 0, "--no-pack-iram",      &options.no_pack_iram, "Deprecated: Tells the linker not to pack variables in internal ram"},
     { 0, "--stack-10bit",       &options.stack10bit, "use the 10bit stack for ds400 (default)" },
     { 0, "--use-accelerator",   &options.useAccelerator, "generate code for ds400 arithmetic accelerator"},
     { 0, "--protect-sp-update", &options.protect_sp_update, "will disable interrupts during ESP:SP updates"},
@@ -1507,7 +1511,7 @@ _ds400_finaliseOptions (void)
     }
 
   // hackhack: we're a superset of the 390.
-  addSet(&preArgvSet, Safe_strdup("-DSDCC_ds390"));
+  addSet(&preArgvSet, Safe_strdup("-D__SDCC_ds390"));
   addSet(&preArgvSet, Safe_strdup("-D__ds390"));
 
   /* Hack-o-matic: if we are using the flat24 model,
@@ -1530,8 +1534,9 @@ _ds400_finaliseOptions (void)
     }
   else
     {
-      port->s.fptr_size = 3;
-      port->s.gptr_size = 4;
+      port->s.far_ptr_size = 3;
+      port->s.funcptr_size = 3;
+      port->s.ptr_size = 4;
 
       port->stack.isr_overhead += 2;      /* Will save dpx on ISR entry. */
 
@@ -1556,12 +1561,6 @@ _ds400_finaliseOptions (void)
           // assumes IDM1:0 = 1:0, CMA = 1.
         }
 
-      /* generate native code 16*16 mul/div */
-      if (options.useAccelerator)
-        port->support.muldiv=2;
-      else
-        port->support.muldiv=1;
-
       /* Fixup the memory map for the stack; it is now in
        * far space and requires a FPOINTER to access it.
        */
@@ -1635,9 +1634,10 @@ PORT ds400_port =
     NULL,
     NULL,
     NULL,
+    NULL,
   },
-  /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-  { 1, 2, 2, 4, 8, 1, 2, 3, 1, 4, 4 },
+  /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float, _BitInt (in bits) */
+  { 1, 2, 2, 4, 8, 1, 2, 3, 2, 3, 1, 4, 64 },
 
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },           /* far, near, xstack, code */
@@ -1666,12 +1666,14 @@ PORT ds400_port =
     NULL,                       // name of segment for copies of initialized variables in code space
     NULL,
     NULL,
+    1,
+    true,                       // unqualified pointer can point to __sfr: TODO: CHECK IF THIS IS ACTUALLY SUPPORTED. Set to true to emulate behaviour of rpevious version of sdcc for now.
     1
   },
   { _ds400_generateRomDataArea, _ds400_linkRomDataArea },
-  { +1, 1, 4, 1, 1, 0 },
-  /* ds390 has an 16 bit mul & div */
-  { 2, -1 },
+  0,                            // ABI revision
+  { +1, 1, 4, 1, 1, 0, 0 },
+  { -1, FALSE },
   { ds390_emitDebuggerSymbol },
   {
     255/4,      /* maxCount */
