@@ -105,7 +105,7 @@ hc08_regWithIdx (int idx)
 }
 
 /*-----------------------------------------------------------------*/
-/* hc08_freeReg - frees a register                                      */
+/* hc08_freeReg - frees a register                                 */
 /*-----------------------------------------------------------------*/
 void
 hc08_freeReg (reg_info * reg)
@@ -138,12 +138,14 @@ hc08_freeReg (reg_info * reg)
       case HX_IDX:
         hc08_reg_h->isFree = 1;
         hc08_reg_x->isFree = 1;
+        hc08_reg_hx->isFree = 1;
         if (hc08_reg_a->isFree)
           hc08_reg_xa->isFree = 1;
         break;
       case XA_IDX:
         hc08_reg_x->isFree = 1;
         hc08_reg_a->isFree = 1;
+        hc08_reg_xa->isFree = 1;
         if (hc08_reg_h->isFree)
           hc08_reg_hx->isFree = 1;
         break;
@@ -280,7 +282,7 @@ DEFSETFUNC (isFree)
   /* if it is free && and the itmp assigned to
      this does not have any overlapping live ranges
      with the one currently being assigned and
-     the size can be accomodated  */
+     the size can be accommodated  */
   if (sym->isFree &&
       noOverLap (sym->usl.itmpStack, fsym) &&
       getSize (sym->type) >= getSize (fsym->type))
@@ -765,66 +767,6 @@ createRegMask (eBBlock ** ebbs, int count)
     }
 }
 
-/*-----------------------------------------------------------------*/
-/* rematStr - returns the rematerialized string for a remat var    */
-/*-----------------------------------------------------------------*/
-static char *
-rematStr (symbol * sym, int offset)
-{
-  iCode *ic = sym->rematiCode;
-
-  while (1)
-    {
-      /* if plus adjust offset to right hand side */
-      if (ic->op == '+')
-        {
-          offset += (int) operandLitValue (IC_RIGHT (ic));
-          ic = OP_SYMBOL (IC_LEFT (ic))->rematiCode;
-          continue;
-        }
-
-      /* if minus adjust offset to right hand side */
-      if (ic->op == '-')
-        {
-          offset -= (int) operandLitValue (IC_RIGHT (ic));
-          ic = OP_SYMBOL (IC_LEFT (ic))->rematiCode;
-          continue;
-        }
-
-      /* cast then continue */
-      if (IS_CAST_ICODE(ic))
-        {
-          ic = OP_SYMBOL (IC_RIGHT (ic))->rematiCode;
-          continue;
-        }
-      /* we reached the end */
-      break;
-    }
-
-  if (ic->op == ADDRESS_OF)
-    {
-      if (offset)
-        {
-          SNPRINTF (buffer, sizeof(buffer),
-                    "(%s %c 0x%04x)",
-                    OP_SYMBOL (IC_LEFT (ic))->rname,
-                    offset >= 0 ? '+' : '-',
-                    abs (offset) & 0xffff);
-        }
-      else
-        {
-          strncpyz (buffer, OP_SYMBOL (IC_LEFT (ic))->rname, sizeof(buffer));
-        }
-    }
-  else if (ic->op == '=')
-    {
-      offset += (int) operandLitValue (IC_RIGHT (ic));
-      SNPRINTF (buffer, sizeof(buffer),
-                "0x%04x",
-                offset & 0xffff);
-    }
-  return buffer;
-}
 
 /*-----------------------------------------------------------------*/
 /* regTypeNum - computes the type & number of registers required   */
@@ -834,7 +776,6 @@ regTypeNum (eBBlock *ebbs)
 {
   symbol *sym;
   int k;
-  iCode *ic;
 
   /* for each live range do */
   for (sym = hTabFirstItem (liveRanges, &k); sym;
@@ -856,34 +797,13 @@ regTypeNum (eBBlock *ebbs)
           if (sym->ruonly || sym->accuse)
             {
               if (IS_AGGREGATE (sym->type) || sym->isptr)
-                sym->type = aggrToPtr (sym->type, FALSE);
+                sym->type = aggrToPtr (sym->type, false);
               continue;
             }
 
-          /* if the symbol has only one definition &
-             that definition is a get_pointer */
-          if (bitVectnBitsOn (sym->defs) == 1 &&
-              (ic = hTabItemWithKey (iCodehTab, bitVectFirstBit (sym->defs))) &&
-              POINTER_GET (ic) &&
-              !IS_BITVAR (sym->etype) &&
-              (aggrToPtrDclType (operandType (IC_LEFT (ic)), FALSE) == POINTER))
-            {
-              if (ptrPseudoSymSafe (sym, ic))
-                {
-                  int rightval = 0;
-                  if (IC_RIGHT (ic))
-                    rightval = (int)operandLitValue (IC_RIGHT (ic));
-                  ptrPseudoSymConvert (sym, ic, rematStr (OP_SYMBOL (IC_LEFT (ic)), rightval));
-                  continue;
-                }
-
-              /* if in data space or idata space then try to
-                 allocate pointer register */
-            }
-
           /* if not then we require registers */
           sym->nRegs = ((IS_AGGREGATE (sym->type) || sym->isptr) ?
-                        getSize (sym->type = aggrToPtr (sym->type, FALSE)) :
+                        getSize (sym->type = aggrToPtr (sym->type, false)) :
                         getSize (sym->type));
 
           if (sym->nRegs > 8)
@@ -1102,16 +1022,6 @@ packRegsForAssign (iCode * ic, eBBlock * ebp)
     }
 
   /* found the definition */
-  /* replace the result with the result of */
-  /* this assignment and remove this assignment */
-  bitVectUnSetBit (OP_SYMBOL (IC_RESULT (dic))->defs, dic->key);
-  ReplaceOpWithCheaperOp (&IC_RESULT (dic), IC_RESULT (ic));
-
-  if (IS_ITEMP (IC_RESULT (dic)) && OP_SYMBOL (IC_RESULT (dic))->liveFrom > dic->seq)
-    {
-      OP_SYMBOL (IC_RESULT (dic))->liveFrom = dic->seq;
-    }
-  // TODO: and the otherway around?
 
   /* delete from liverange table also
      delete from all the points inbetween and the new
@@ -1123,6 +1033,17 @@ packRegsForAssign (iCode * ic, eBBlock * ebp)
         bitVectSetBit (sic->rlive, IC_RESULT (dic)->key);
     }
 
+  /* replace the result with the result of */
+  /* this assignment and remove this assignment */
+  bitVectUnSetBit (OP_SYMBOL (IC_RESULT (dic))->defs, dic->key);
+  ReplaceOpWithCheaperOp (&IC_RESULT (dic), IC_RESULT (ic));
+
+  if (IS_ITEMP (IC_RESULT (dic)) && OP_SYMBOL (IC_RESULT (dic))->liveFrom > dic->seq)
+    {
+      OP_SYMBOL (IC_RESULT (dic))->liveFrom = dic->seq;
+    }
+  // TODO: and the otherway around?
+
   remiCodeFromeBBlock (ebp, ic);
   bitVectUnSetBit (OP_DEFS (IC_RESULT (ic)), ic->key);
   hTabDeleteItem (&iCodehTab, ic->key, ic, DELETE_ITEM, NULL);
@@ -1181,11 +1102,11 @@ findAssignToSym (operand * op, iCode * ic)
     return NULL;
 
   /* if the symbol is volatile then we should not */
-  if (isOperandVolatile (IC_RIGHT (dic), TRUE))
+  if (isOperandVolatile (IC_RIGHT (dic), true))
     return NULL;
-  /* XXX TODO --- should we be passing FALSE to isOperandVolatile()?
+  /* XXX TODO --- should we be passing false to isOperandVolatile()?
      What does it mean for an iTemp to be volatile, anyway? Passing
-     TRUE is more cautious but may prevent possible optimizations */
+     true is more cautious but may prevent possible optimizations */
 
   /* if the symbol is in far space then we should not */
   /* if (isOperandInFarSpace (IC_RIGHT (dic)))
@@ -1237,7 +1158,7 @@ reassignAliasedSym (eBBlock *ebp, iCode *assignment, iCode *use, operand *op)
 
   /* update the sym's liverange */
   if ( OP_LIVETO(op) < ic->seq )
-    setToRange(op, ic->seq, FALSE);
+    setToRange(op, ic->seq, false);
 
   /* remove the assignment iCode now that its result is unused */
   remiCodeFromeBBlock (ebp, assignment);
@@ -1289,162 +1210,6 @@ packRegsForSupport (iCode * ic, eBBlock * ebp)
   return changes;
 }
 
-
-#if 0
-/*-----------------------------------------------------------------*/
-/* packRegsForOneuse : - will reduce some registers for single Use */
-/*-----------------------------------------------------------------*/
-static iCode *
-packRegsForOneuse (iCode * ic, operand * op, eBBlock * ebp)
-{
-  bitVect *uses;
-  iCode *dic, *sic;
-
-  /* if returning a literal then do nothing */
-  if (!IS_SYMOP (op))
-    return NULL;
-
-  /* only up to 2 bytes */
-  if (getSize (operandType (op)) > (fReturnSizeHC08 - 2))
-    return NULL;
-
-  return NULL;
-
-  if (ic->op != SEND && //RETURN
-      ic->op != SEND &&
-      !POINTER_SET (ic) &&
-      !POINTER_GET (ic))
-    return NULL;
-
-  if (ic->op == SEND && ic->argreg != 1)
-    return NULL;
-
-  /* this routine will mark the symbol as used in one
-     instruction use only && if the definition is local
-     (ie. within the basic block) && has only one definition &&
-     that definition is either a return value from a
-     function or does not contain any variables in
-     far space */
-  uses = bitVectCopy (OP_USES (op));
-  bitVectUnSetBit (uses, ic->key);      /* take away this iCode */
-  if (!bitVectIsZero (uses))    /* has other uses */
-    return NULL;
-
-  /* if it has only one definition */
-  if (bitVectnBitsOn (OP_DEFS (op)) > 1)
-    return NULL;                /* has more than one definition */
-
-  /* get that definition */
-  if (!(dic = hTabItemWithKey (iCodehTab, bitVectFirstBit (OP_DEFS (op)))))
-    return NULL;
-
-  /* if that only usage is a cast */
-  if (dic->op == CAST)
-    {
-      /* to a bigger type */
-      if (getSize(OP_SYM_TYPE(IC_RESULT(dic))) > getSize(OP_SYM_TYPE(IC_RIGHT(dic))))
-        {
-          /* then we can not, since we cannot predict the usage of b & acc */
-          return NULL;
-        }
-    }
-
-  /* found the definition now check if it is local */
-  if (dic->seq < ebp->fSeq || dic->seq > ebp->lSeq)
-    return NULL;                /* non-local */
-
-  /* now check if it is the return from a function call */
-  if (dic->op == CALL || dic->op == PCALL)
-    {
-      if (ic->op != SEND && ic->op != RETURN &&
-          !POINTER_SET(ic) && !POINTER_GET(ic))
-        {
-          OP_SYMBOL (op)->ruonly = 1;
-          return dic;
-        }
-      dic = dic->next;
-    }
-
-
-//  /* otherwise check that the definition does
-//     not contain any symbols in far space */
-//  if (isOperandInFarSpace (IC_LEFT (dic)) ||
-//      isOperandInFarSpace (IC_RIGHT (dic)) ||
-//      IS_OP_RUONLY (IC_LEFT (ic)) ||
-//      IS_OP_RUONLY (IC_RIGHT (ic)))
-//    {
-//      return NULL;
-//    }
-
-  /* if pointer set then make sure the pointer
-     is one byte */
-#if 0
-  if (POINTER_SET (dic) &&
-      !IS_DATA_PTR (aggrToPtr (operandType (IC_RESULT (dic)), FALSE)))
-    return NULL;
-
-  if (POINTER_GET (dic) &&
-      !IS_DATA_PTR (aggrToPtr (operandType (IC_LEFT (dic)), FALSE)))
-    return NULL;
-#endif
-
-  sic = dic;
-
-  /* make sure the intervening instructions
-     don't have anything in far space */
-  for (dic = dic->next; dic && dic != ic && sic != ic; dic = dic->next)
-    {
-      /* if there is an intervening function call then no */
-      if (dic->op == CALL || dic->op == PCALL)
-        return NULL;
-      /* if pointer set then make sure the pointer
-         is one byte */
-#if 0
-      if (POINTER_SET (dic) &&
-          !IS_DATA_PTR (aggrToPtr (operandType (IC_RESULT (dic)), FALSE)))
-        return NULL;
-
-      if (POINTER_GET (dic) &&
-          !IS_DATA_PTR (aggrToPtr (operandType (IC_LEFT (dic)), FALSE)))
-        return NULL;
-#endif
-      /* if address of & the result is remat then okay */
-      if (dic->op == ADDRESS_OF &&
-          OP_SYMBOL (IC_RESULT (dic))->remat)
-        continue;
-
-      /* if operand has size of three or more & this
-         operation is a '*','/' or '%' then 'b' may
-         cause a problem */
-#if 0
-      if ((dic->op == '%' || dic->op == '/' || dic->op == '*') &&
-          getSize (operandType (op)) >= 3)
-        return NULL;
-#endif
-
-      /* if left or right or result is in far space */
-//      if (isOperandInFarSpace (IC_LEFT (dic)) ||
-//        isOperandInFarSpace (IC_RIGHT (dic)) ||
-//        isOperandInFarSpace (IC_RESULT (dic)) ||
-//        IS_OP_RUONLY (IC_LEFT (dic)) ||
-//        IS_OP_RUONLY (IC_RIGHT (dic)) ||
-//        IS_OP_RUONLY (IC_RESULT (dic)))
-//      {
-//        return NULL;
-//      }
-//      /* if left or right or result is on stack */
-//     if (isOperandOnStack(IC_LEFT(dic)) ||
-//        isOperandOnStack(IC_RIGHT(dic)) ||
-//        isOperandOnStack(IC_RESULT(dic))) {
-//      return NULL;
-//     }
-    }
-
-  OP_SYMBOL (op)->ruonly = 1;
-  return sic;
-}
-#endif
-
 /*-----------------------------------------------------------------*/
 /* isBitwiseOptimizable - requirements of JEAN LOUIS VERN          */
 /*-----------------------------------------------------------------*/
@@ -1469,59 +1234,9 @@ isBitwiseOptimizable (iCode * ic)
   */
   if (IS_LITERAL(rtype) ||
       (IS_BITVAR (ltype) && IN_BITSPACE (SPEC_OCLS (ltype))))
-    return TRUE;
+    return true;
   else
-    return FALSE;
-}
-
-/*-----------------------------------------------------------------*/
-/* isCommutativeOp - tests whether this op cares what order its    */
-/*                   operands are in                               */
-/*-----------------------------------------------------------------*/
-bool isCommutativeOp2(unsigned int op)
-{
-  if (op == '+' || op == '*' || op == EQ_OP ||
-      op == '^' || op == '|' || op == BITWISEAND)
-    return TRUE;
-  else
-    return FALSE;
-}
-
-/*-----------------------------------------------------------------*/
-/* operandUsesAcc2 - determines whether the code generated for this */
-/*                  operand will have to use the accumulator       */
-/*-----------------------------------------------------------------*/
-bool operandUsesAcc2(operand *op)
-{
-  if (!op)
-    return FALSE;
-
-  if (IS_SYMOP(op)) {
-    symbol *sym = OP_SYMBOL(op);
-    memmap *symspace;
-
-    if (sym->accuse)
-      return TRUE;  /* duh! */
-
-    if (IS_ITEMP(op))
-      {
-        if (SPIL_LOC(op)) {
-          sym = SPIL_LOC(op);  /* if spilled, look at spill location */
-        } else {
-          return FALSE;  /* more checks? */
-        }
-      }
-
-    symspace = SPEC_OCLS(sym->etype);
-
-    if (IN_BITSPACE(symspace))
-      return TRUE;  /* fetching bit vars uses the accumulator */
-
-    if (IN_FARSPACE(symspace) || IN_CODESPACE(symspace))
-      return TRUE;  /* fetched via accumulator and dptr */
-  }
-
-  return FALSE;
+    return false;
 }
 
 /*-----------------------------------------------------------------*/
@@ -1685,12 +1400,12 @@ packRegsForAccUse (iCode * ic)
 /* packForPush - heuristics to reduce iCode for pushing            */
 /*-----------------------------------------------------------------*/
 static void
-packForPush (iCode * ic, eBBlock ** ebpp, int blockno)
+packForPush (iCode * ic, eBBlock ** ebpp, int count)
 {
   iCode *dic, *lic;
   bitVect *dbv;
-  struct eBBlock * ebp=ebpp[blockno];
   int disallowHiddenAssignment = 0;
+  eBBlock * ebp = ebpp[ic->eBBlockNum];
 
   if ((ic->op != IPUSH && ic->op != SEND) || !IS_ITEMP (IC_LEFT (ic)))
     return;
@@ -1708,10 +1423,10 @@ packForPush (iCode * ic, eBBlock ** ebpp, int blockno)
   if (dic->op != '=' || POINTER_SET (dic))
     return;
 
-  if (dic->seq < ebp->fSeq) // Evelyn did this
+  if (dic->seq < ebp->fSeq || dic->seq > ebp->lSeq) // Evelyn did this
     {
       int i;
-      for (i=0; i<blockno; i++)
+      for (i=0; i<count; i++)
         {
           if (dic->seq >= ebpp[i]->fSeq && dic->seq <= ebpp[i]->lSeq)
             {
@@ -1719,7 +1434,8 @@ packForPush (iCode * ic, eBBlock ** ebpp, int blockno)
               break;
             }
         }
-      wassert (i!=blockno); // no way to recover from here
+      if (i==count) // Abort if we can't find the definition's block
+        return;
     }
 
   if (IS_SYMOP(IC_RIGHT(dic)))
@@ -1918,21 +1634,12 @@ packPointerOp (iCode * ic, eBBlock ** ebpp)
     }
 
   /* Put the remaining operand on the right and convert to assignment     */
-  /* or cast (Sometimes the operands to the addition are different sizes, */
-  /* so there is an implicit cast. If so, need to make it explicit so     */
-  /* that all the bytes of the pointer are defined. */
   if (IS_SYMOP (offsetOp))
     bitVectUnSetBit (OP_USES (offsetOp), dic->key);
   IC_RIGHT (dic) = nonOffsetOp;
   IC_LEFT (dic) = NULL;
   SET_ISADDR (IC_RESULT (dic), 0);
-  if (getSize (operandType (pointer)) == getSize (operandType (nonOffsetOp)))
-    dic->op = '=';
-  else
-    {
-      dic->op = CAST;
-      IC_LEFT (dic) = operandFromLink (operandType (pointer));
-    }
+  dic->op = '=';
 }
 
 /*-----------------------------------------------------------------*/
@@ -1940,140 +1647,149 @@ packPointerOp (iCode * ic, eBBlock ** ebpp)
 /*                   pressure                                      */
 /*-----------------------------------------------------------------*/
 static void
-packRegisters (eBBlock ** ebpp, int blockno)
+packRegisters (eBBlock ** ebpp, int count)
 {
   iCode *ic;
   int change = 0;
-  eBBlock *ebp = ebpp[blockno];
+  int blockno;
 
-  do
+  for (blockno=0; blockno<count; blockno++)
     {
-      change = 0;
+      eBBlock *ebp = ebpp[blockno];
+
+      do
+        {
+          change = 0;
+
+          /* look for assignments of the form */
+          /* iTempNN = TrueSym (someoperation) SomeOperand */
+          /*       ....                       */
+          /* TrueSym := iTempNN:1             */
+          for (ic = ebp->sch; ic; ic = ic->next)
+            {
+              /* find assignment of the form TrueSym := iTempNN:1 */
+              if (ic->op == '=' && !POINTER_SET (ic))
+                change += packRegsForAssign (ic, ebp);
+            }
+        }
+      while (change);
 
-      /* look for assignments of the form */
-      /* iTempNN = TrueSym (someoperation) SomeOperand */
-      /*       ....                       */
-      /* TrueSym := iTempNN:1             */
       for (ic = ebp->sch; ic; ic = ic->next)
         {
-          /* find assignment of the form TrueSym := iTempNN:1 */
-          if (ic->op == '=' && !POINTER_SET (ic))
-            change += packRegsForAssign (ic, ebp);
-        }
-    }
-  while (change);
-
-  for (ic = ebp->sch; ic; ic = ic->next)
-    {
-      //packRegsForLiteral (ic);
+          //packRegsForLiteral (ic);
       
-      /* move SEND to immediately precede its CALL/PCALL */
-      if (ic->op == SEND && ic->next &&
-          ic->next->op != CALL && ic->next->op != PCALL)
-        {
-          ic = moveSendToCall (ic, ebp);
-        }
+          /* move SEND to immediately precede its CALL/PCALL */
+          if (ic->op == SEND && ic->next &&
+              ic->next->op != CALL && ic->next->op != PCALL)
+            {
+              ic = moveSendToCall (ic, ebp);
+            }
       
-      /* if this is an itemp & result of an address of a true sym
-         then mark this as rematerialisable   */
-      if (ic->op == ADDRESS_OF &&
-          IS_ITEMP (IC_RESULT (ic)) &&
-          IS_TRUE_SYMOP (IC_LEFT (ic)) &&
-          bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 &&
-          !OP_SYMBOL (IC_LEFT (ic))->onStack)
-        {
-          OP_SYMBOL (IC_RESULT (ic))->remat = 1;
-          OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
-          OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
-        }
-
-      /* if straight assignment then carry remat flag if
-         this is the only definition */
-      if (ic->op == '=' &&
-          !POINTER_SET (ic) &&
-          IS_SYMOP (IC_RIGHT (ic)) &&
-          OP_SYMBOL (IC_RIGHT (ic))->remat &&
-          bitVectnBitsOn (OP_SYMBOL (IC_RESULT (ic))->defs) <= 1 &&
-          !OP_SYMBOL (IC_RESULT (ic))->_isparm)
-        {
-          OP_SYMBOL (IC_RESULT (ic))->remat = OP_SYMBOL (IC_RIGHT (ic))->remat;
-          OP_SYMBOL (IC_RESULT (ic))->rematiCode = OP_SYMBOL (IC_RIGHT (ic))->rematiCode;
-        }
-
-      /* if cast to a generic pointer & the pointer being
-         cast is remat, then we can remat this cast as well */
-      if (ic->op == CAST &&
-          IS_SYMOP(IC_RIGHT(ic)) &&
-          OP_SYMBOL(IC_RIGHT(ic))->remat &&
-          bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 &&
-          !OP_SYMBOL (IC_RESULT (ic))->_isparm)
-        {
-          sym_link *to_type = operandType(IC_LEFT(ic));
-          sym_link *from_type = operandType(IC_RIGHT(ic));
-          if (IS_PTR(to_type) && IS_PTR(from_type))
+          /* if this is an itemp & result of an address of a true sym
+             then mark this as rematerialisable   */
+          if (ic->op == ADDRESS_OF &&
+              IS_ITEMP (IC_RESULT (ic)) &&
+              IS_TRUE_SYMOP (IC_LEFT (ic)) &&
+              bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 &&
+              !OP_SYMBOL (IC_LEFT (ic))->onStack)
             {
               OP_SYMBOL (IC_RESULT (ic))->remat = 1;
               OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
               OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
             }
+
+          /* if straight assignment then carry remat flag if
+             this is the only definition */
+          if (ic->op == '=' &&
+              !POINTER_SET (ic) &&
+              IS_SYMOP (IC_RIGHT (ic)) &&
+              OP_SYMBOL (IC_RIGHT (ic))->remat &&
+              bitVectnBitsOn (OP_SYMBOL (IC_RESULT (ic))->defs) <= 1 &&
+              !OP_SYMBOL (IC_RESULT (ic))->_isparm &&
+              !OP_SYMBOL (IC_RESULT (ic))->addrtaken &&
+              !isOperandGlobal (IC_RESULT (ic)))
+            {
+              OP_SYMBOL (IC_RESULT (ic))->remat = OP_SYMBOL (IC_RIGHT (ic))->remat;
+              OP_SYMBOL (IC_RESULT (ic))->rematiCode = OP_SYMBOL (IC_RIGHT (ic))->rematiCode;
+            }
+
+          /* if cast to a generic pointer & the pointer being
+             cast is remat, then we can remat this cast as well */
+          if (ic->op == CAST &&
+              IS_SYMOP(IC_RIGHT(ic)) &&
+              OP_SYMBOL(IC_RIGHT(ic))->remat &&
+              bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 &&
+              !OP_SYMBOL (IC_RESULT (ic))->_isparm &&
+              !OP_SYMBOL (IC_RESULT (ic))->addrtaken &&
+              !isOperandGlobal (IC_RESULT (ic)))
+            {
+              sym_link *to_type = operandType(IC_LEFT(ic));
+              sym_link *from_type = operandType(IC_RIGHT(ic));
+              if (IS_PTR(to_type) && IS_PTR(from_type))
+                {
+                  OP_SYMBOL (IC_RESULT (ic))->remat = 1;
+                  OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
+                  OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
+                }
+            }
+
+          /* if this is a +/- operation with a rematerizable
+             then mark this as rematerializable as well */
+          if ((ic->op == '+' || ic->op == '-') &&
+              (IS_SYMOP (IC_LEFT (ic)) &&
+               IS_ITEMP (IC_RESULT (ic)) &&
+               IS_OP_LITERAL (IC_RIGHT (ic))) &&
+               OP_SYMBOL (IC_LEFT (ic))->remat &&
+              (!IS_SYMOP (IC_RIGHT (ic)) || !IS_CAST_ICODE(OP_SYMBOL (IC_RIGHT (ic))->rematiCode)) &&
+               bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1)
+            {
+              OP_SYMBOL (IC_RESULT (ic))->remat = 1;
+              OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
+              OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
+            }
+          /* mark the pointer usages */
+          if (POINTER_SET (ic) && IS_SYMOP (IC_RESULT (ic)))
+            OP_SYMBOL (IC_RESULT (ic))->uptr = 1;
+
+          if (POINTER_GET (ic) && IS_SYMOP (IC_LEFT (ic)))
+            OP_SYMBOL (IC_LEFT (ic))->uptr = 1;
+
+          /* reduce for support function calls */
+          if (ic->supportRtn || (ic->op != IFX && ic->op != JUMPTABLE))
+            packRegsForSupport (ic, ebp);
+
+          /* if the condition of an if instruction
+             is defined in the previous instruction and
+             this is the only usage then
+             mark the itemp as a conditional */
+          if ((IS_CONDITIONAL (ic) ||
+               (IS_BITWISE_OP(ic) && isBitwiseOptimizable (ic))) &&
+              ic->next && ic->next->op == IFX &&
+              bitVectnBitsOn (OP_USES(IC_RESULT(ic)))==1 &&
+              isOperandEqual (IC_RESULT (ic), IC_COND (ic->next)) &&
+              OP_SYMBOL (IC_RESULT (ic))->liveTo <= ic->next->seq)
+            {
+              OP_SYMBOL (IC_RESULT (ic))->regType = REG_CND;
+              continue;
+            }
+
+          /* pack for PUSH
+             iTempNN := (some variable in farspace) V1
+             push iTempNN ;
+             -------------
+             push V1
+           */
+          if (ic->op == IPUSH || ic->op == SEND)
+            {
+              packForPush (ic, ebpp, count);
+            }
+
+          if (POINTER_SET (ic) || POINTER_GET (ic))
+            packPointerOp (ic, ebpp);
+
+          if (options.oldralloc)
+            packRegsForAccUse (ic);
         }
-
-      /* if this is a +/- operation with a rematerizable
-         then mark this as rematerializable as well */
-      if ((ic->op == '+' || ic->op == '-') &&
-          (IS_SYMOP (IC_LEFT (ic)) &&
-           IS_ITEMP (IC_RESULT (ic)) &&
-           IS_OP_LITERAL (IC_RIGHT (ic))) &&
-           OP_SYMBOL (IC_LEFT (ic))->remat &&
-          (!IS_SYMOP (IC_RIGHT (ic)) || !IS_CAST_ICODE(OP_SYMBOL (IC_RIGHT (ic))->rematiCode)) &&
-           bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1)
-        {
-          OP_SYMBOL (IC_RESULT (ic))->remat = 1;
-          OP_SYMBOL (IC_RESULT (ic))->rematiCode = ic;
-          OP_SYMBOL (IC_RESULT (ic))->usl.spillLoc = NULL;
-        }
-      /* mark the pointer usages */
-      if (POINTER_SET (ic) && IS_SYMOP (IC_RESULT (ic)))
-        OP_SYMBOL (IC_RESULT (ic))->uptr = 1;
-
-      if (POINTER_GET (ic) && IS_SYMOP (IC_LEFT (ic)))
-        OP_SYMBOL (IC_LEFT (ic))->uptr = 1;
-
-      /* reduce for support function calls */
-      if (ic->supportRtn || (ic->op != IFX && ic->op != JUMPTABLE))
-        packRegsForSupport (ic, ebp);
-
-      /* if the condition of an if instruction
-         is defined in the previous instruction and
-         this is the only usage then
-         mark the itemp as a conditional */
-      if ((IS_CONDITIONAL (ic) ||
-           (IS_BITWISE_OP(ic) && isBitwiseOptimizable (ic))) &&
-          ic->next && ic->next->op == IFX &&
-          bitVectnBitsOn (OP_USES(IC_RESULT(ic)))==1 &&
-          isOperandEqual (IC_RESULT (ic), IC_COND (ic->next)) &&
-          OP_SYMBOL (IC_RESULT (ic))->liveTo <= ic->next->seq)
-        {
-          OP_SYMBOL (IC_RESULT (ic))->regType = REG_CND;
-          continue;
-        }
-
-      /* pack for PUSH
-         iTempNN := (some variable in farspace) V1
-         push iTempNN ;
-         -------------
-         push V1
-       */
-      if (ic->op == IPUSH || ic->op == SEND)
-        {
-          packForPush (ic, ebpp, blockno);
-        }
-
-      if (POINTER_SET (ic) || POINTER_GET (ic))
-        packPointerOp (ic, ebpp);
-
-      if (options.oldralloc)
-        packRegsForAccUse (ic);
     }
 }
 
@@ -2197,7 +1913,6 @@ hc08_oldralloc (ebbIndex * ebbi)
   eBBlock ** ebbs = ebbi->bbOrder;
   int count = ebbi->count;
   iCode *ic;
-  int i;
 
   setToNull ((void *) &_G.funcrUsed);
   setToNull ((void *) &_G.regAssigned);
@@ -2215,12 +1930,11 @@ hc08_oldralloc (ebbIndex * ebbi)
   /* change assignments this will remove some
      live ranges reducing some register pressure */
 
-  for (i = 0; i < count; i++)
-    packRegisters (ebbs, i);
+  packRegisters (ebbs, count);
 
   /* liveranges probably changed by register packing
      so we compute them again */
-  recomputeLiveRanges (ebbs, count, FALSE);
+  recomputeLiveRanges (ebbs, count, false);
 
   if (options.dump_i_code)
     dumpEbbsToFileExt (DUMP_PACK, ebbi);
@@ -2398,6 +2112,7 @@ serialRegMark (eBBlock ** ebbs, int count)
         }
     }
 }
+
 /*-----------------------------------------------------------------*/
 /* New register allocator                                          */
 /*-----------------------------------------------------------------*/
@@ -2407,7 +2122,6 @@ hc08_ralloc (ebbIndex * ebbi)
   eBBlock ** ebbs = ebbi->bbOrder;
   int count = ebbi->count;
   iCode *ic;
-  int i;
 
   setToNull ((void *) &_G.funcrUsed);
   setToNull ((void *) &_G.regAssigned);
@@ -2425,12 +2139,11 @@ hc08_ralloc (ebbIndex * ebbi)
   /* change assignments this will remove some
      live ranges reducing some register pressure */
 
-  for (i = 0; i < count; i++)
-    packRegisters (ebbs, i);
+  packRegisters (ebbs, count);
 
   /* liveranges probably changed by register packing
      so we compute them again */
-  recomputeLiveRanges (ebbs, count, FALSE);
+  recomputeLiveRanges (ebbs, count, false);
 
   if (options.dump_i_code)
     dumpEbbsToFileExt (DUMP_PACK, ebbi);
