@@ -26,10 +26,17 @@
 #include "dbuf_string.h"
 
 #include "ralloc.h"
+#include "gen.h"
+#include "peep.h"
 
 extern DEBUGFILE dwarf2DebugFile;
 extern int dwarf2FinalizeFile(FILE *);
 
+static char pdk_defaultRules[] = {
+#include "peeph.rul"
+  ""
+};
+
 static char *pdk_keywords[] = {
   "at",
   "code",
@@ -80,8 +87,11 @@ pdk_genInitStartup (FILE *of)
   fprintf (of, "p::\n");
   fprintf (of, "\t.ds 2\n");
   
+  // WARNING: program memory address 0x20:21 (0x10 in words) is reserved for the
+  // global singular interrupt vector. Ensure startup code never grows too large
+  // and intrudes on this location.
 
-  fprintf (of, "\t.area\tHEADER (ABS)\n"); // In the header we have 16 bytes. First should be nop.
+  fprintf (of, "\t.area\tHEADER (ABS)\n");
   fprintf (of, "\t.org 0x0000\n");
   fprintf (of, "\tnop\n"); // First word is a jump to self-test routine at end of ROM on some new devices.
 
@@ -92,21 +102,34 @@ pdk_genInitStartup (FILE *of)
   if (options.stack_loc >= 0)
     {
       fprintf (of, "\tmov\ta, #0x%02x\n", options.stack_loc);
-      fprintf (of, "\tmov\tsp, a\n");
+      fprintf (of, "\tmov.io\tsp, a\n");
     }
   else
     {
       fprintf (of, "\tmov\ta, #s_OSEG\n");
       fprintf (of, "\tadd\ta, #l_OSEG + 1\n");
       fprintf (of, "\tand\ta, #0xfe\n");
-      fprintf (of, "\tmov\tsp, a\n");
+      fprintf (of, "\tmov.io\tsp, a\n");
     }
 
-  fprintf (of, "\tcall\t__sdcc_external_startup\n");
-  fprintf (of, "\tgoto\t__sdcc_gs_init_startup\n");
+  /* Skip init if __sdcc_external_startup returned non-zero */
+  fprintf (of, "\tcall\t___sdcc_external_startup\n");
+  if (TARGET_IS_PDK13)
+    {
+      // No "cneqsn" instruction on pdk13 - we must invert the logic.
+      fprintf (of, "\tnot\ta\n");
+      fprintf (of, "\tceqsn\ta, #0xFF\n");
+      fprintf (of, "\tgoto\t__sdcc_program_startup\n");
+      fprintf (of, "\tgoto\t__sdcc_init_data\n");
+    }
+  else
+    {
+      fprintf (of, "\tcneqsn\ta, #0x00\n");
+      fprintf (of, "\tgoto\t__sdcc_init_data\n");
+      fprintf (of, "\tgoto\t__sdcc_program_startup\n");
+    }
 
   tfprintf (of, "\t!area\n", STATIC_NAME);
-  fprintf (of, "__sdcc_gs_init_startup:\n");
 
   /* Init static & global variables */
   fprintf (of, "__sdcc_init_data:\n");
@@ -114,22 +137,24 @@ pdk_genInitStartup (FILE *of)
   /* Zeroing memory (required by standard for static & global variables) */
   fprintf (of, "\tmov\ta, #s_DATA\n");
   fprintf (of, "\tmov\tp, a\n");
-  fprintf (of, "00001$:\n");
-  fprintf (of, "\tmov\ta, #s_DATA\n");
-  fprintf (of, "\tadd\ta, #l_DATA\n");
-  fprintf (of, "\tcneqsn\ta, p\n");
   fprintf (of, "\tgoto\t00002$\n");
+  fprintf (of, "00001$:\n");
   fprintf (of, "\tmov\ta, #0x00\n");
   fprintf (of, "\tidxm\tp, a\n");
   fprintf (of, "\tinc\tp\n");
-  fprintf (of, "\tgoto\t00001$\n");
+  fprintf (of, "\tmov\ta, #s_DATA\n");
   fprintf (of, "00002$:\n");
+  fprintf (of, "\tadd\ta, #l_DATA\n");
+  fprintf (of, "\tceqsn\ta, p\n");
+  fprintf (of, "\tgoto\t00001$\n");
 }
 
 static void
 pdk_init (void)
 {
   asm_addTree (&asm_asxxxx_smallpdk_mapping);
+
+  pdk_init_asmops();
 }
 
 static void
@@ -187,6 +212,9 @@ _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
   if (ic->op != '*')
     return (false);
 
+  if (IS_BITINT (OP_SYM_TYPE (IC_RESULT(ic))) && SPEC_BITINTWIDTH (OP_SYM_TYPE (IC_RESULT(ic))) % 8)
+    return false;
+
   return ((IS_LITERAL (left) || IS_LITERAL (right)) && result_size == 1);
 }
 
@@ -194,7 +222,13 @@ _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
 static bool
 hasExtBitOp (int op, int size)
 {
-  return (false);
+  return (op == GETBYTE || op == SWAP && size == 1);
+}
+
+static const char *
+get_model (void)
+{
+    return(options.stackAuto ? "pdk15-stack-auto" : "pdk15");
 }
 
 /** $1 is always the basename.
@@ -211,7 +245,7 @@ static const char *_linkCmd[] =
 /* $3 is replaced by assembler.debug_opts resp. port->assembler.plain_opts */
 static const char *pdk13AsmCmd[] =
 {
-  "sdaspdk14", "$l", "$3", "\"$1.asm\"", NULL
+  "sdaspdk13", "$l", "$3", "\"$1.asm\"", NULL
 };
 
 static const char *const _libs_pdk13[] = { "pdk13", NULL, };
@@ -247,12 +281,14 @@ PORT pdk13_port =
     _libs_pdk13,                /* libs */
   },
   {                             /* Peephole optimizer */
+    pdk_defaultRules,
+    pdkinstructionSize,
     0,
     0,
     0,
+    pdknotUsed,
     0,
-    0,
-    0,
+    pdknotUsedFrom,
     0,
     0,
     0,
@@ -271,6 +307,7 @@ PORT pdk13_port =
     0,                          /* banked func ptr */
     1,                          /* bit */
     4,                          /* float */
+    64,                         /* bit-precise integer types up to _BitInt (64) */
   },
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
@@ -299,9 +336,11 @@ PORT pdk13_port =
     0,
     0,
     1,                          /* CODE  is read-only */
+    false,                      // no instructions accessing i/o space with indirect addressing available in hardware
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
+  0,                            /* ABI revision */
   {                             /* stack information */
      +1,                        /* direction: stack grows up */
      0,
@@ -312,7 +351,7 @@ PORT pdk13_port =
      1,                         /* sp points to next free stack location */
   },     
   { -1, false },                /* no int x int -> long multiplication support routine. */
-  { 0,
+  { pdk_emitDebuggerSymbol,
     {
       0,
       0,                        /* cfiSame */
@@ -413,14 +452,14 @@ PORT pdk14_port =
     _libs_pdk14,                /* libs */
   },
   {                             /* Peephole optimizer */
+    pdk_defaultRules,
+    pdkinstructionSize,
     0,
     0,
     0,
+    pdknotUsed,
     0,
-    0,
-    0,
-    0,
-    0,
+    pdknotUsedFrom,
     0,
   },
   /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
@@ -437,6 +476,7 @@ PORT pdk14_port =
     0,                          /* banked func ptr */
     1,                          /* bit */
     4,                          /* float */
+    64,                         /* bit-precise integer types up to _BitInt (64) */
   },
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
@@ -465,9 +505,11 @@ PORT pdk14_port =
     0,
     0,
     1,                          /* CODE  is read-only */
+    false,                      // no instructions accessing i/o space with indirect addressing available in hardware
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
+  0,                            /* ABI revision */
   {                             /* stack information */
      +1,                        /* direction: stack grows up */
      0,
@@ -478,7 +520,7 @@ PORT pdk14_port =
      1,                         /* sp points to next free stack location */
   },     
   { -1, false },                /* no int x int -> long multiplication support routine. */
-  { 0,
+  { pdk_emitDebuggerSymbol,
     {
       0,
       0,                        /* cfiSame */
@@ -559,7 +601,7 @@ PORT pdk15_port =
     true,
     NO_MODEL,
     NO_MODEL,
-    0,                          /* model == target */
+    &get_model,
   },
   {                             /* Assembler */
     pdk15AsmCmd,
@@ -579,14 +621,14 @@ PORT pdk15_port =
     _libs_pdk15,                /* libs */
   },
   {                             /* Peephole optimizer */
+    pdk_defaultRules,
+    pdkinstructionSize,
     0,
     0,
     0,
+    pdknotUsed,
     0,
-    0,
-    0,
-    0,
-    0,
+    pdknotUsedFrom,
     0,
   },
   /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
@@ -603,6 +645,7 @@ PORT pdk15_port =
     0,                          /* banked func ptr */
     1,                          /* bit */
     4,                          /* float */
+    64,                         /* bit-precise integer types up to _BitInt (64) */
   },
   /* tags for generic pointers */
   { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
@@ -631,9 +674,11 @@ PORT pdk15_port =
     0,
     0,
     1,                          /* CODE  is read-only */
+    false,                      // no instructions accessing i/o space with indirect addressing available in hardware
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
+  0,                            /* ABI revision */
   {                             /* stack information */
      +1,                        /* direction: stack grows up */
      0,
@@ -644,7 +689,7 @@ PORT pdk15_port =
      1,                         /* sp points to next free stack location */
   },     
   { -1, false },                /* no int x int -> long multiplication support routine. */
-  { 0,
+  { pdk_emitDebuggerSymbol,
     {
       0,
       0,                        /* cfiSame */
