@@ -84,6 +84,7 @@ pic16_sectioninfo_t pic16_sectioninfo;
 int has_xinst_config = 0;
 
 static int regParmFlg = 0;  /* determine if we can register a parameter */
+static struct sym_link *regParmFuncType;
 
 pic16_options_t pic16_options;
 pic16_config_options_t *pic16_config_options;
@@ -113,11 +114,15 @@ static void
 _pic16_reset_regparm (struct sym_link *funcType)
 {
   regParmFlg = 0;
+  regParmFuncType = funcType;
 }
 
 static int
 _pic16_regparm (sym_link * l, bool reentrant)
 {
+  if (IFFUNC_HASVARARGS (regParmFuncType))
+    return 0;
+
   /* force all parameters via SEND/RECEIVE */
   if(0 /*pic16_options.ip_stack*/) {
     /* for this processor it is simple
@@ -1229,7 +1234,6 @@ hasExtBitOp (int op, int size)
   if (op == RRC
       || op == RLC
       || op == GETABIT
-      /* || op == GETHBIT */ /* GETHBIT doesn't look complete for PIC */
      )
     return TRUE;
   else
@@ -1311,12 +1315,14 @@ PORT pic16_port =
     2,      /* int */
     4,      /* long */
     8,      /* long long */
-    2,      /* ptr */
-    3,      /* fptr, far pointers (see Microchip) */
+    2,      /* near ptr */
+    3,      /* far ptr, far pointers (see Microchip) */
     3,      /* gptr */
+    3,      /* func ptr */
+    3,      /* banked func ptr */
     1,      /* bit */
     4,      /* float */
-    4       /* max */
+    0,      /* _BitInt (in bits) */
   },
 
     /* generic pointer tags */
@@ -1352,12 +1358,14 @@ PORT pic16_port =
     NULL,                   // default location for auto vars
     NULL,                   // default location for global vars
     1,                      // code is read only 1=yes
+    true,                  // unqualified pointer can point to __sfr: TODO: CHECK IF THIS IS ACTUALLY SUPPORTED. Set to true to emulate behaviour of rpevious version of sdcc for now.
     1                       // No fancy alignments supported.
   },
   {
     NULL,       /* genExtraAreaDeclaration */
     NULL        /* genExatrAreaLinkOptions */
   },
+  0,            /* ABI revision */
   {
     /* stack related information */
     -1,         /* -1 stack grows downwards, +1 upwards */
@@ -1365,11 +1373,11 @@ PORT pic16_port =
     4,          /* extra overhead when the function is an ISR */
     1,          /* extra overhead for a function call */
     1,          /* re-entrant space */
-    0           /* 'banked' call overhead, mild overlap with bank_overhead */
+    0,          /* 'banked' call overhead, mild overlap with bank_overhead */
+    1           /* sp is offset by 1 from last item pushed */
   },
-    /* pic16 has an 8 bit mul */
   {
-     0, -1
+     -1, FALSE
   },
   {
     pic16_emitDebuggerSymbol
