@@ -1,6 +1,8 @@
 /** setjmp/longjmp tests.
 */
 #include <testfwk.h>
+
+#if !defined(__SDCC_pic14) // Unimplemented setjmp
 #include <setjmp.h>
 
 unsigned int global_int = 0;
@@ -10,19 +12,13 @@ unsigned int *gpInt;
 #include <8052.h>
 
 void
-T2_isr (void) __interrupt 5 //no using
+T2_isr (void) __interrupt (5) //no using
 {
   //do not clear flag TF2 so it keeps interrupting !
   (*gpInt)++;
 }
 #endif
 
-#if defined(__SDCC_pic14)
-#define SKIP
-#endif
-
-#ifndef SKIP
-
 void
 try_fun (jmp_buf catch, int except)
 {
@@ -40,21 +36,26 @@ void g(void)
 void f1(void)
 {
 	static int i;
-	int j = setjmp(buf);
+	int j;
+	i= 0;
+	j= setjmp(buf);
 	ASSERT(i == j);
 	i++;
 	if(!j)
 		g();
 }
 
-#else
-#warning Skipped setjmp/longjmp test
+#endif
+
+// Get FreeBSD version to skip part of test for known broken setjmp (FreeBSD bug #255320, affecting at least FreeBSD 13.0 and FreeBSD 13.1).
+#ifdef __FreeBSD__
+#include <sys/param.h>
 #endif
 
 void
 testJmp (void)
 {
-#ifndef SKIP
+#if !defined(__SDCC_pic14) // Unimplemented setjmp
   jmp_buf catch;
   int exception;
 
@@ -74,9 +75,11 @@ testJmp (void)
       ASSERT (0);
     }
   ASSERT (exception == 1);
-
+  
+#if !defined(__FreeBSD__) || __FreeBSD_version > 1301000
   f1();
 #endif
+#endif
 
 // C99 might require setjmp to be a macro. The standard seems self-contradicting on this issue.
 //#ifndef setjmp
